@@ -3,7 +3,6 @@
 
 #include "vm/types.h"
 
-#include <ucontext.h>	/* for gregset_t */
 #include <stdbool.h>
 #include <limits.h>
 
@@ -27,8 +26,10 @@ enum machine_reg {
 	MACH_REG_XMM6,
 	MACH_REG_XMM7,
 
+	NR_FP_REGISTERS,
+
 	/* The above registers are available for register allocator.  */
-	NR_REGISTERS,
+	NR_REGISTERS = NR_FP_REGISTERS,
 
 	MACH_REG_EBP = NR_REGISTERS,
 	MACH_REG_ESP,
@@ -46,7 +47,7 @@ enum machine_reg {
 	MACH_REG_xBP = MACH_REG_EBP,
 	MACH_REG_xSP = MACH_REG_ESP,
 
-	MACH_REG_UNASSIGNED = INT_MAX,
+	MACH_REG_UNASSIGNED = CHAR_MAX,
 };
 
 #define GPR_VM_TYPE	J_INT
@@ -56,7 +57,12 @@ extern enum machine_reg caller_save_regs[NR_CALLER_SAVE_REGS];
 
 const char *reg_name(enum machine_reg reg);
 
-bool reg_supports_type(enum machine_reg reg, enum vm_type type);
+extern uint32_t reg_type_table[NR_REGISTERS];
+
+static inline bool reg_supports_type(enum machine_reg reg, enum vm_type type)
+{
+	return reg_type_table[reg] & (1UL << type);
+}
 
 struct register_state {
 	uint64_t			ip;
@@ -73,21 +79,14 @@ struct register_state {
 	};
 };
 
-static inline void
-save_signal_registers(struct register_state *regs, gregset_t gregs)
-{
-	regs->ip	= (uint32_t) gregs[REG_EIP];
-	regs->eax	= gregs[REG_EAX];
-	regs->ebx	= gregs[REG_EBX];
-	regs->ecx	= gregs[REG_ECX];
-	regs->edx	= gregs[REG_EDX];
-	regs->esi	= gregs[REG_ESI];
-	regs->edi	= gregs[REG_EDI];
-}
-
 static inline enum vm_type reg_default_type(enum machine_reg reg)
 {
 	return GPR_VM_TYPE;
 }
 
+static inline bool is_xmm_reg(enum machine_reg reg)
+{
+	return reg >= NR_GP_REGISTERS && reg < NR_FP_REGISTERS;
+}
+
 #endif /* X86_REGISTERS_32_H */
