@@ -3,7 +3,6 @@
 
 #include "vm/types.h"
 
-#include <ucontext.h>  /* for gregset_t */
 #include <stdbool.h>
 #include <limits.h>
 
@@ -26,7 +25,6 @@ enum machine_reg {
 	/* Number of general purpose registers.  */
 	NR_GP_REGISTERS,
 
-
 	MACH_REG_XMM0,
 	MACH_REG_XMM1,
 	MACH_REG_XMM2,
@@ -65,7 +63,7 @@ enum machine_reg {
 	MACH_REG_xBP = MACH_REG_RBP,
 	MACH_REG_xSP = MACH_REG_RSP,
 
-	MACH_REG_UNASSIGNED = INT_MAX,
+	MACH_REG_UNASSIGNED = CHAR_MAX,
 };
 
 #define GPR_VM_TYPE	J_LONG
@@ -101,26 +99,6 @@ struct register_state {
 	};
 };
 
-static inline void
-save_signal_registers(struct register_state *regs, gregset_t gregs)
-{
-	regs->ip	= gregs[REG_RIP];
-        regs->rax	= gregs[REG_RAX];
-        regs->rbx	= gregs[REG_RBX];
-        regs->rcx	= gregs[REG_RCX];
-        regs->rdx	= gregs[REG_RDX];
-        regs->rsi	= gregs[REG_RSI];
-        regs->rdi	= gregs[REG_RDI];
-        regs->r8	= gregs[REG_R8];
-        regs->r9	= gregs[REG_R9];
-        regs->r10	= gregs[REG_R10];
-        regs->r11	= gregs[REG_R11];
-        regs->r12	= gregs[REG_R12];
-        regs->r13	= gregs[REG_R13];
-        regs->r14	= gregs[REG_R14];
-        regs->r15	= gregs[REG_R15];
-}
-
 static inline enum vm_type reg_default_type(enum machine_reg reg)
 {
 	if (reg < NR_GP_REGISTERS)
@@ -131,4 +109,9 @@ static inline enum vm_type reg_default_type(enum machine_reg reg)
 		return GPR_VM_TYPE;
 }
 
+static inline bool is_xmm_reg(enum machine_reg reg)
+{
+	return reg >= NR_GP_REGISTERS && reg < NR_FP_REGISTERS;
+}
+
 #endif /* X86_REGISTERS_64_H */
