@@ -29,8 +29,11 @@
 #include "jit/instruction.h"
 #include "jit/vars.h"
 
+#include "lib/arena.h"
+
 #include <stdlib.h>
 #include <string.h>
+#include <assert.h>
 
 struct insn *alloc_insn(enum insn_type type)
 {
@@ -38,7 +41,6 @@ struct insn *alloc_insn(enum insn_type type)
 	if (insn) {
 		memset(insn, 0, sizeof *insn);
 		INIT_LIST_HEAD(&insn->insn_list_node);
-		INIT_LIST_HEAD(&insn->branch_list_node);
 		insn->type = type;
 	}
 	return insn;
@@ -73,6 +75,17 @@ void free_insn(struct insn *insn)
 	free(insn);
 }
 
+void free_ssa_insn(struct insn *insn)
+{
+	unsigned long ndx;
+	release_operand(&insn->ssa_dest);
+
+	for (ndx = 0; ndx < insn->nr_srcs; ndx++)
+		release_operand(&insn->ssa_srcs[ndx]);
+
+	free(insn);
+}
+
 static void init_membase_operand(struct insn *insn, struct operand *operand,
 				 struct var_info *base_reg, unsigned long disp)
 {
@@ -105,6 +118,16 @@ static void init_reg_operand(struct insn *insn, struct operand *operand, struct
 	operand->reg.kind = insn_operand_use_kind(insn, operand);
 }
 
+static void init_phi_reg_operand(struct insn *insn, struct operand *operand, struct var_info *reg)
+{
+	operand->type = OPERAND_REG;
+
+	init_register(&operand->reg, insn, reg->interval);
+	if (operand == &insn->ssa_dest)
+		operand->reg.kind = USE_KIND_OUTPUT;
+	else operand->reg.kind = USE_KIND_INPUT;
+}
+
 struct insn *insn(enum insn_type insn_type)
 {
 	return alloc_insn(insn_type);
@@ -367,6 +390,23 @@ struct insn *rel_insn(enum insn_type insn_type, unsigned long rel)
 	return insn;
 }
 
+struct insn *phi_insn(enum insn_type insn_type, struct var_info *var, unsigned long nr_srcs)
+{
+	struct insn *insn = alloc_insn(insn_type);
+	unsigned long ndx;
+
+	if (insn) {
+		init_phi_reg_operand(insn, &insn->ssa_dest, var);
+
+		insn->ssa_srcs = malloc(nr_srcs * sizeof(struct operand));
+		for (ndx = 0; ndx < nr_srcs; ndx++)
+			init_phi_reg_operand(insn, &insn->ssa_srcs[ndx], var);
+
+		insn->nr_srcs = nr_srcs;
+	}
+	return insn;
+}
+
 struct insn *branch_insn(enum insn_type insn_type, struct basic_block *if_true)
 {
 	struct insn *insn = alloc_insn(insn_type);
@@ -441,6 +481,24 @@ struct insn *imm_memlocal_insn(enum insn_type insn_type,
 	return insn;
 }
 
+struct insn *ic_call_insn(struct var_info *this, unsigned long imm)
+{
+	struct insn *insn = alloc_insn(INSN_IC_CALL);
+
+	if (insn) {
+		init_reg_operand(insn, &insn->src, this);
+		assert(insn_operand_use_kind(insn, &insn->src) == USE_KIND_INPUT);
+		insn->dest	= (struct operand) {
+			.type		= OPERAND_IMM,
+			{
+				.imm		= imm,
+			}
+		};
+	}
+
+	return insn;
+}
+
 int insert_copy_slot_32_insns(struct stack_slot *from, struct stack_slot *to,
 			      struct list_head *add_before, unsigned long bc_offset)
 {
@@ -483,7 +541,7 @@ int insert_copy_slot_64_insns(struct stack_slot *from, struct stack_slot *to,
 	if (!push_hi)
 		goto fail_push_hi;
 
-	push_lo = memlocal_insn(INSN_PUSH_MEMLOCAL, get_next_slot(from));
+	push_lo = memlocal_insn(INSN_PUSH_MEMLOCAL, from->next);
 	if (!push_lo)
 		goto fail_push_lo;
 
@@ -491,7 +549,7 @@ int insert_copy_slot_64_insns(struct stack_slot *from, struct stack_slot *to,
 	if (!pop_hi)
 		goto fail_pop_hi;
 
-	pop_lo = memlocal_insn(INSN_POP_MEMLOCAL, get_next_slot(to));
+	pop_lo = memlocal_insn(INSN_POP_MEMLOCAL, to->next);
 	if (!pop_lo)
 		goto fail_pop_lo;
 
@@ -527,6 +585,53 @@ int insert_copy_slot_64_insns(struct stack_slot *from, struct stack_slot *to,
 
 #endif  /* CONFIG_X86_32 */
 
+struct insn *ssa_reg_reg_insn(struct var_info *src, struct var_info *dest)
+{
+	struct insn *insn;
+
+	switch(src->vm_type) {
+	case J_FLOAT:
+		insn = reg_reg_insn(INSN_MOVSS_XMM_XMM, src, dest);
+		break;
+	case J_DOUBLE:
+		insn = reg_reg_insn(INSN_MOVSD_XMM_XMM, src, dest);
+		break;
+	default:
+		insn = reg_reg_insn(INSN_MOV_REG_REG, src, dest);
+	}
+
+	return insn;
+}
+
+struct insn *ssa_imm_reg_insn(unsigned long imm,
+			struct var_info *dest_reg,
+			struct var_info *gpr,
+			struct insn **insn_conv)
+{
+	struct insn *insn;
+
+	switch(dest_reg->vm_type) {
+	case J_FLOAT:
+		insn = imm_reg_insn(INSN_MOV_IMM_REG, imm, gpr);
+		*insn_conv = reg_reg_insn(INSN_CONV_GPR_TO_FPU, gpr, dest_reg);
+		break;
+	case J_DOUBLE:
+		insn = imm_reg_insn(INSN_MOV_IMM_REG, imm, gpr);
+		*insn_conv = reg_reg_insn(INSN_CONV_GPR_TO_FPU64, gpr, dest_reg);
+		break;
+	default:
+		insn = imm_reg_insn(INSN_MOV_IMM_REG, imm, dest_reg);
+		*insn_conv = NULL;
+	}
+
+	return insn;
+}
+
+struct insn *ssa_phi_insn(struct var_info *var, unsigned long nr_srcs)
+{
+	return phi_insn(INSN_PHI, var, nr_srcs);
+}
+
 struct insn *spill_insn(struct var_info *var, struct stack_slot *slot)
 {
 	enum insn_type insn_type;
@@ -572,6 +677,89 @@ struct insn *jump_insn(struct basic_block *bb)
 	return branch_insn(INSN_JMP_BRANCH, bb);
 }
 
+void ssa_chg_jmp_direction(struct insn *insn, struct basic_block *after_bb,
+		struct basic_block *new_bb, struct basic_block *bb)
+{
+	switch (insn->type) {
+	case INSN_JE_BRANCH:
+		if (insn->operand.branch_target == bb) {
+			insn->operand.branch_target = after_bb;
+			insn->type = INSN_JNE_BRANCH;
+		}
+		break;
+	case INSN_JNE_BRANCH:
+		if (insn->operand.branch_target == bb) {
+			insn->operand.branch_target = after_bb;
+			insn->type = INSN_JE_BRANCH;
+		}
+		break;
+	case INSN_JGE_BRANCH:
+		if (insn->operand.branch_target == bb) {
+			insn->operand.branch_target = after_bb;
+			insn->type = INSN_JL_BRANCH;
+		}
+		break;
+	case INSN_JL_BRANCH:
+		if (insn->operand.branch_target == bb) {
+			insn->operand.branch_target = after_bb;
+			insn->type = INSN_JGE_BRANCH;
+		}
+		break;
+	case INSN_JG_BRANCH:
+		if (insn->operand.branch_target == bb) {
+			insn->operand.branch_target = after_bb;
+			insn->type = INSN_JLE_BRANCH;
+		}
+		break;
+	case INSN_JLE_BRANCH:
+		if (insn->operand.branch_target == bb) {
+			insn->operand.branch_target = after_bb;
+			insn->type = INSN_JG_BRANCH;
+		}
+		break;
+	case INSN_JMP_BRANCH:
+		if (insn->operand.branch_target == bb)
+			insn->operand.branch_target = new_bb;
+		break;
+	}
+}
+
+int ssa_modify_insn_type(struct insn *insn)
+{
+	switch(insn->type) {
+	case INSN_MOV_REG_MEMBASE:
+		insn->type = INSN_MOV_IMM_MEMBASE;
+		break;
+
+	case INSN_MOV_REG_REG:
+		insn->type = INSN_MOV_IMM_REG;
+		break;
+
+	case INSN_MOV_REG_THREAD_LOCAL_MEMBASE:
+		insn->type = INSN_MOV_IMM_THREAD_LOCAL_MEMBASE;
+		break;
+
+	case INSN_MOV_REG_MEMLOCAL:
+		insn->type = INSN_MOV_IMM_MEMLOCAL;
+		break;
+
+	default:
+		return -1;
+	}
+
+	return 0;
+}
+
+void imm_operand(struct operand *operand, unsigned long imm)
+{
+	*operand = (struct operand) {
+		.type = OPERAND_IMM,
+			{
+				.imm = imm,
+			}
+	};
+}
+
 /*
  *	Instruction flags
  */
@@ -642,6 +830,7 @@ static unsigned long insn_flags[] = {
 	[INSN_FSTP_64_MEMLOCAL]			= USE_FP | DEF_NONE,
 	[INSN_FSTP_MEMBASE]			= USE_SRC | DEF_NONE,
 	[INSN_FSTP_MEMLOCAL]			= USE_FP | DEF_NONE,
+	[INSN_IC_CALL]				= USE_SRC | DEF_xAX | DEF_xCX | TYPE_CALL,
 	[INSN_JE_BRANCH]			= USE_NONE | DEF_NONE | TYPE_BRANCH,
 	[INSN_JGE_BRANCH]			= USE_NONE | DEF_NONE | TYPE_BRANCH,
 	[INSN_JG_BRANCH]			= USE_NONE | DEF_NONE | TYPE_BRANCH,
@@ -700,6 +889,7 @@ static unsigned long insn_flags[] = {
 	[INSN_OR_IMM_MEMBASE]			= USE_DST | DEF_NONE,
 	[INSN_OR_MEMBASE_REG]			= USE_SRC | USE_DST | DEF_DST,
 	[INSN_OR_REG_REG]			= USE_SRC | USE_DST | DEF_DST,
+	[INSN_PHI]				= USE_SRC | DEF_DST,
 	[INSN_POP_MEMLOCAL]			= USE_SRC | DEF_NONE,
 	[INSN_POP_REG]				= USE_NONE | DEF_DST,
 	[INSN_PUSH_IMM]				= USE_NONE | DEF_NONE,
@@ -747,6 +937,41 @@ static struct mach_reg_def checkregs[] = {
 	{ MACH_REG_xDX, DEF_xDX },
 };
 
+int insn_use_def(struct insn *insn)
+{
+	if (insn_use_def_dst(insn))
+		return 1;
+
+	if (insn_use_def_src(insn))
+		return 1;
+
+	return 0;
+}
+
+int insn_use_def_dst(struct insn *insn)
+{
+	unsigned long flags;
+
+	flags = insn_flags[insn->type];
+
+	if ((flags & DEF_DST) && (flags & USE_DST))
+		return 1;
+
+	return 0;
+}
+
+int insn_use_def_src(struct insn *insn)
+{
+	unsigned long flags;
+
+	flags = insn_flags[insn->type];
+
+	if ((flags & DEF_SRC) && (flags & USE_SRC))
+		return 1;
+
+	return 0;
+}
+
 int insn_defs(struct compilation_unit *cu, struct insn *insn, struct var_info **defs)
 {
 	unsigned long flags;
@@ -767,6 +992,30 @@ int insn_defs(struct compilation_unit *cu, struct insn *insn, struct var_info **
 	return nr;
 }
 
+bool insn_vreg_def(struct use_position *use, struct var_info *var)
+{
+	return use->kind == USE_KIND_OUTPUT && use->interval->var_info == var;
+}
+
+int insn_defs_reg(struct insn *insn, struct use_position **regs)
+{
+	unsigned long flags;
+	int nr = 0;
+
+	flags = insn_flags[insn->type];
+
+	if (flags & DEF_SRC)
+		regs[nr++] = &insn->src.reg;
+
+	if (flags & DEF_DST) {
+		if (insn->type != INSN_PHI)
+			regs[nr++] = &insn->dest.reg;
+		else regs[nr++] = &insn->ssa_dest.reg;
+	}
+
+	return nr;
+}
+
 int insn_uses(struct insn *insn, struct var_info **uses)
 {
 	unsigned long flags;
@@ -789,6 +1038,45 @@ int insn_uses(struct insn *insn, struct var_info **uses)
 	return nr;
 }
 
+bool insn_vreg_use(struct use_position *use, struct var_info *var)
+{
+	if (use->kind == USE_KIND_INVALID)
+		return true;
+
+	return use->kind == USE_KIND_INPUT && use->interval->var_info == var;
+}
+
+unsigned long insn_uses_reg(struct insn *insn, struct use_position **regs)
+{
+	unsigned long flags;
+	unsigned long nr = 0;
+
+	flags = insn_flags[insn->type];
+
+	if (insn_is_phi(insn)) {
+		if (flags & USE_SRC) {
+			for (unsigned long i = 0; i < insn->nr_srcs; i++)
+				regs[nr++] = &insn->ssa_srcs[i].reg;
+		}
+
+		return nr;
+	}
+
+	if (flags & USE_SRC)
+		regs[nr++] = &insn->src.reg;
+
+	if (flags & USE_DST)
+		regs[nr++] = &insn->dest.reg;
+
+	if (flags & USE_IDX_SRC)
+		regs[nr++] = &insn->src.index_reg;
+
+	if (flags & USE_IDX_DST)
+		regs[nr++] = &insn->dest.index_reg;
+
+	return nr;
+}
+
 int insn_operand_use_kind(struct insn *insn, struct operand *operand)
 {
 	unsigned long flags;
@@ -817,6 +1105,11 @@ int insn_operand_use_kind(struct insn *insn, struct operand *operand)
 	return kind_mask;
 }
 
+bool insn_is_mov_imm_reg(struct insn *insn)
+{
+	return insn->type == INSN_MOV_IMM_REG;
+}
+
 bool insn_is_branch(struct insn *insn)
 {
 	unsigned long flags = insn_flags[insn->type];
@@ -824,9 +1117,21 @@ bool insn_is_branch(struct insn *insn)
 	return flags & TYPE_BRANCH;
 }
 
-bool insn_is_call(struct insn *insn)
+bool insn_is_jmp_mem(struct insn *insn)
 {
-	unsigned long flags = insn_flags[insn->type];
+	if (!insn)
+		return false;
 
-	return flags & TYPE_CALL;
+	if (insn->type == INSN_JMP_MEMINDEX || insn->type == INSN_JMP_MEMBASE)
+		return true;
+
+	return false;
+}
+
+unsigned long nr_srcs_phi(struct insn *insn)
+{
+	if (!insn_is_phi(insn))
+		return 0;
+
+	return insn->nr_srcs;
 }
