@@ -119,9 +119,9 @@ static int print_imm_memlocal(struct string *str, struct insn *insn)
 
 static int print_imm_memdisp(struct string *str, struct insn *insn)
 {
-	print_imm(str, &insn->operands[0]);
+	print_imm(str, &insn->src);
 	str_append(str, ", ");
-	return print_memdisp(str, &insn->operands[1]);
+	return print_memdisp(str, &insn->dest);
 }
 
 static int print_membase_reg(struct string *str, struct insn *insn)
@@ -232,6 +232,15 @@ static int print_reg_reg(struct string *str, struct insn *insn)
 	return print_reg(str, &insn->dest);
 }
 
+static int print_reg_regs(struct string *str, struct insn *insn)
+{
+	for (unsigned long ndx = 0; ndx < insn->nr_srcs; ndx++) {
+		print_reg(str, &insn->ssa_srcs[ndx]);
+		str_append(str, ", ");
+	}
+	return print_reg(str, &insn->ssa_dest);
+}
+
 #define print_func_name(str) str_append(str, "%-20s ", __func__ + 6)
 
 static int print_adc_imm_reg(struct string *str, struct insn *insn)
@@ -246,6 +255,18 @@ static int print_adc_membase_reg(struct string *str, struct insn *insn)
 	return print_membase_reg(str, insn);
 }
 
+static int print_addss_xmm_xmm(struct string *str, struct insn *insn)
+{
+	print_func_name(str);
+	return print_reg_reg(str, insn);
+}
+
+static int print_addsd_xmm_xmm(struct string *str, struct insn *insn)
+{
+	print_func_name(str);
+	return print_reg_reg(str, insn);
+}
+
 static int print_adc_reg_reg(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
@@ -270,61 +291,49 @@ static int print_add_reg_reg(struct string *str, struct insn *insn)
 	return print_reg_reg(str, insn);
 }
 
-static int print_fadd_reg_reg(struct string *str, struct insn *insn)
-{
-	print_func_name(str);
-	return print_reg_reg(str, insn);
-}
-
-static int print_fadd_64_reg_reg(struct string *str, struct insn *insn)
-{
-	print_func_name(str);
-	return print_reg_reg(str, insn);
-}
-
-static int print_fadd_64_memdisp_reg(struct string *str, struct insn *insn)
+static int print_addsd_memdisp_reg(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memdisp_reg(str, insn);
 }
 
-static int print_fsub_reg_reg(struct string *str, struct insn *insn)
+static int print_subss_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
 }
 
-static int print_fsub_64_reg_reg(struct string *str, struct insn *insn)
+static int print_subsd_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
 }
 
-static int print_fmul_reg_reg(struct string *str, struct insn *insn)
+static int print_mulss_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
 }
 
-static int print_fmul_64_reg_reg(struct string *str, struct insn *insn)
+static int print_mulsd_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
 }
 
-static int print_fmul_64_memdisp_reg(struct string *str, struct insn *insn)
+static int print_fmul_64_memdisp_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memdisp_reg(str, insn);
 }
 
-static int print_fdiv_reg_reg(struct string *str, struct insn *insn)
+static int print_divss_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
 }
 
-static int print_fdiv_64_reg_reg(struct string *str, struct insn *insn)
+static int print_divsd_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
@@ -378,6 +387,15 @@ static int print_fstp_memlocal(struct string *str, struct insn *insn)
 	return print_memlocal(str, &insn->operand);
 }
 
+static int print_ic_call(struct string *str, struct insn *insn)
+{
+	print_func_name(str);
+	print_reg(str, &insn->src);
+	str_append(str, ", ");
+	print_imm(str, &insn->dest);
+	return str_append(str, "<%s>", ((struct vm_method *)insn->dest.imm)->name);
+}
+
 static int print_fstp_64_memlocal(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
@@ -418,7 +436,7 @@ static int print_call_reg(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	str_append(str, "(");
-	print_reg(str, &insn->operand);
+	print_reg(str, &insn->dest);
 	return str_append(str, ")");
 }
 
@@ -464,25 +482,25 @@ static int print_div_reg_reg(struct string *str, struct insn *insn)
 	return print_reg_reg(str, insn);
 }
 
-static int print_mov_membase_xmm(struct string *str, struct insn *insn)
+static int print_movss_membase_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_membase_reg(str, insn);
 }
 
-static int print_mov_64_membase_xmm(struct string *str, struct insn *insn)
+static int print_movsd_membase_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_membase_reg(str, insn);
 }
 
-static int print_mov_xmm_membase(struct string *str, struct insn *insn)
+static int print_movss_xmm_membase(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_membase(str, insn);
 }
 
-static int print_mov_64_xmm_membase(struct string *str, struct insn *insn)
+static int print_movsd_xmm_membase(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_membase(str, insn);
@@ -563,13 +581,13 @@ static int print_jmp_branch(struct string *str, struct insn *insn)
 static int print_jmp_memindex(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
-	return print_memindex(str, &insn->operand);
+	return print_memindex(str, &insn->dest);
 }
 
 static int print_jmp_membase(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
-	return print_membase(str, &insn->operand);
+	return print_membase(str, &insn->dest);
 }
 
 static int print_jne_branch(struct string *str, struct insn *insn)
@@ -602,13 +620,13 @@ static int print_mov_memlocal_reg(struct string *str, struct insn *insn)
 	return print_memlocal_reg(str, insn);
 }
 
-static int print_mov_memlocal_xmm(struct string *str, struct insn *insn)
+static int print_movss_memlocal_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memlocal_reg(str, insn);
 }
 
-static int print_mov_64_memlocal_xmm(struct string *str, struct insn *insn)
+static int print_movsd_memlocal_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memlocal_reg(str, insn);
@@ -626,13 +644,13 @@ static int print_mov_memdisp_reg(struct string *str, struct insn *insn)
 	return print_memdisp_reg(str, insn);
 }
 
-static int print_mov_memdisp_xmm(struct string *str, struct insn *insn)
+static int print_movss_memdisp_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memdisp_reg(str, insn);
 }
 
-static int print_mov_64_memdisp_xmm(struct string *str, struct insn *insn)
+static int print_movsd_memdisp_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memdisp_reg(str, insn);
@@ -644,13 +662,13 @@ static int print_mov_reg_memdisp(struct string *str, struct insn *insn)
 	return print_reg_memdisp(str, insn);
 }
 
-static int print_mov_xmm_memdisp(struct string *str, struct insn *insn)
+static int print_movss_xmm_memdisp(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_memdisp(str, insn);
 }
 
-static int print_mov_64_xmm_memdisp(struct string *str, struct insn *insn)
+static int print_movsd_xmm_memdisp(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_memdisp(str, insn);
@@ -686,13 +704,13 @@ static int print_mov_memindex_reg(struct string *str, struct insn *insn)
 	return print_memindex_reg(str, insn);
 }
 
-static int print_mov_memindex_xmm(struct string *str, struct insn *insn)
+static int print_movss_memindex_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memindex_reg(str, insn);
 }
 
-static int print_mov_64_memindex_xmm(struct string *str, struct insn *insn)
+static int print_movsd_memindex_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_memindex_reg(str, insn);
@@ -710,13 +728,13 @@ static int print_mov_reg_memindex(struct string *str, struct insn *insn)
 	return print_reg_memindex(str, insn);
 }
 
-static int print_mov_xmm_memindex(struct string *str, struct insn *insn)
+static int print_movss_xmm_memindex(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_memindex(str, insn);
 }
 
-static int print_mov_64_xmm_memindex(struct string *str, struct insn *insn)
+static int print_movsd_xmm_memindex(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_memindex(str, insn);
@@ -728,13 +746,13 @@ static int print_mov_reg_memlocal(struct string *str, struct insn *insn)
 	return print_reg_memlocal(str, insn);
 }
 
-static int print_mov_xmm_memlocal(struct string *str, struct insn *insn)
+static int print_movss_xmm_memlocal(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_memlocal(str, insn);
 }
 
-static int print_mov_64_xmm_memlocal(struct string *str, struct insn *insn)
+static int print_movsd_xmm_memlocal(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_memlocal(str, insn);
@@ -746,13 +764,13 @@ static int print_mov_reg_reg(struct string *str, struct insn *insn)
 	return print_reg_reg(str, insn);
 }
 
-static int print_mov_xmm_xmm(struct string *str, struct insn *insn)
+static int print_movss_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
 }
 
-static int print_mov_64_xmm_xmm(struct string *str, struct insn *insn)
+static int print_movsd_xmm_xmm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
 	return print_reg_reg(str, insn);
@@ -800,7 +818,7 @@ static int print_mul_reg_reg(struct string *str, struct insn *insn)
 static int print_neg_reg(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
-	return print_reg(str, &insn->operand);
+	return print_reg(str, &insn->dest);
 }
 
 static int print_or_imm_membase(struct string *str, struct insn *insn)
@@ -821,6 +839,12 @@ static int print_or_reg_reg(struct string *str, struct insn *insn)
 	return print_reg_reg(str, insn);
 }
 
+static int print_phi(struct string *str, struct insn *insn)
+{
+	print_func_name(str);
+	return print_reg_regs(str, insn);
+}
+
 static int print_push_imm(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
@@ -836,19 +860,19 @@ static int print_push_reg(struct string *str, struct insn *insn)
 static int print_push_memlocal(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
-	return print_memlocal(str, &insn->operand);
+	return print_memlocal(str, &insn->src);
 }
 
 static int print_pop_memlocal(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
-	return print_memlocal(str, &insn->operand);
+	return print_memlocal(str, &insn->src);
 }
 
 static int print_pop_reg(struct string *str, struct insn *insn)
 {
 	print_func_name(str);
-	return print_reg(str, &insn->operand);
+	return print_reg(str, &insn->dest);
 }
 
 static int print_ret(struct string *str, struct insn *insn)
@@ -958,6 +982,9 @@ static print_insn_fn insn_printers[] = {
 	[INSN_ADC_IMM_REG] = print_adc_imm_reg,
 	[INSN_ADC_MEMBASE_REG] = print_adc_membase_reg,
 	[INSN_ADC_REG_REG] = print_adc_reg_reg,
+	[INSN_ADDSD_MEMDISP_XMM] = print_addsd_memdisp_reg,
+	[INSN_ADDSD_XMM_XMM] = print_addsd_xmm_xmm,
+	[INSN_ADDSS_XMM_XMM] = print_addss_xmm_xmm,
 	[INSN_ADD_IMM_REG] = print_add_imm_reg,
 	[INSN_ADD_MEMBASE_REG] = print_add_membase_reg,
 	[INSN_ADD_REG_REG] = print_add_reg_reg,
@@ -969,40 +996,29 @@ static print_insn_fn insn_printers[] = {
 	[INSN_CMP_IMM_REG] = print_cmp_imm_reg,
 	[INSN_CMP_MEMBASE_REG] = print_cmp_membase_reg,
 	[INSN_CMP_REG_REG] = print_cmp_reg_reg,
+	[INSN_CONV_FPU64_TO_GPR] = print_conv_fpu64_to_gpr,
+	[INSN_CONV_FPU_TO_GPR] = print_conv_fpu_to_gpr,
+	[INSN_CONV_GPR_TO_FPU64] = print_conv_gpr_to_fpu64,
+	[INSN_CONV_GPR_TO_FPU] = print_conv_gpr_to_fpu,
+	[INSN_CONV_XMM64_TO_XMM] = print_conv_xmm64_to_xmm,
+	[INSN_CONV_XMM_TO_XMM64] = print_conv_xmm_to_xmm64,
+	[INSN_DIVSD_XMM_XMM] = print_divsd_xmm_xmm,
+	[INSN_DIVSS_XMM_XMM] = print_divss_xmm_xmm,
 	[INSN_DIV_MEMBASE_REG] = print_div_membase_reg,
 	[INSN_DIV_REG_REG] = print_div_reg_reg,
-	[INSN_FADD_REG_REG] = print_fadd_reg_reg,
-	[INSN_FADD_64_REG_REG] = print_fadd_64_reg_reg,
-	[INSN_FADD_64_MEMDISP_REG] = print_fadd_64_memdisp_reg,
-	[INSN_FSUB_REG_REG] = print_fsub_reg_reg,
-	[INSN_FSUB_64_REG_REG] = print_fsub_64_reg_reg,
-	[INSN_FMUL_REG_REG] = print_fmul_reg_reg,
-	[INSN_FMUL_64_REG_REG] = print_fmul_64_reg_reg,
-	[INSN_FMUL_64_MEMDISP_REG] = print_fmul_64_memdisp_reg,
-	[INSN_FDIV_REG_REG] = print_fdiv_reg_reg,
-	[INSN_FDIV_64_REG_REG] = print_fdiv_64_reg_reg,
-	[INSN_FLD_MEMBASE] = print_fld_membase,
-	[INSN_FLD_MEMLOCAL] = print_fld_memlocal,
-	[INSN_FLD_64_MEMBASE] = print_fld_64_membase,
-	[INSN_FLD_64_MEMLOCAL] = print_fld_64_memlocal,
-	[INSN_FLDCW_MEMBASE] = print_fldcw_membase,
 	[INSN_FILD_64_MEMBASE] = print_fild_64_membase,
 	[INSN_FISTP_64_MEMBASE] = print_fistp_64_membase,
+	[INSN_FLDCW_MEMBASE] = print_fldcw_membase,
+	[INSN_FLD_64_MEMBASE] = print_fld_64_membase,
+	[INSN_FLD_64_MEMLOCAL] = print_fld_64_memlocal,
+	[INSN_FLD_MEMBASE] = print_fld_membase,
+	[INSN_FLD_MEMLOCAL] = print_fld_memlocal,
 	[INSN_FNSTCW_MEMBASE] = print_fnstcw_membase,
-	[INSN_FSTP_MEMBASE] = print_fstp_membase,
-	[INSN_FSTP_MEMLOCAL] = print_fstp_memlocal,
 	[INSN_FSTP_64_MEMBASE] = print_fstp_64_membase,
 	[INSN_FSTP_64_MEMLOCAL] = print_fstp_64_memlocal,
-	[INSN_MOV_MEMBASE_XMM] = print_mov_membase_xmm,
-	[INSN_MOV_64_MEMBASE_XMM] = print_mov_64_membase_xmm,
-	[INSN_MOV_XMM_MEMBASE] = print_mov_xmm_membase,
-	[INSN_MOV_64_XMM_MEMBASE] = print_mov_64_xmm_membase,
-	[INSN_CONV_FPU_TO_GPR] = print_conv_fpu_to_gpr,
-	[INSN_CONV_FPU64_TO_GPR] = print_conv_fpu64_to_gpr,
-	[INSN_CONV_GPR_TO_FPU] = print_conv_gpr_to_fpu,
-	[INSN_CONV_GPR_TO_FPU64] = print_conv_gpr_to_fpu64,
-	[INSN_CONV_XMM_TO_XMM64] = print_conv_xmm_to_xmm64,
-	[INSN_CONV_XMM64_TO_XMM] = print_conv_xmm64_to_xmm,
+	[INSN_FSTP_MEMBASE] = print_fstp_membase,
+	[INSN_FSTP_MEMLOCAL] = print_fstp_memlocal,
+	[INSN_IC_CALL] = print_ic_call,
 	[INSN_JE_BRANCH] = print_je_branch,
 	[INSN_JGE_BRANCH] = print_jge_branch,
 	[INSN_JG_BRANCH] = print_jg_branch,
@@ -1012,39 +1028,46 @@ static print_insn_fn insn_printers[] = {
 	[INSN_JMP_MEMBASE] = print_jmp_membase,
 	[INSN_JMP_MEMINDEX] = print_jmp_memindex,
 	[INSN_JNE_BRANCH] = print_jne_branch,
+	[INSN_MOVSD_MEMBASE_XMM] = print_movsd_membase_xmm,
+	[INSN_MOVSD_MEMDISP_XMM] = print_movsd_memdisp_xmm,
+	[INSN_MOVSD_MEMINDEX_XMM] = print_movsd_memindex_xmm,
+	[INSN_MOVSD_MEMLOCAL_XMM] = print_movsd_memlocal_xmm,
+	[INSN_MOVSD_XMM_MEMBASE] = print_movsd_xmm_membase,
+	[INSN_MOVSD_XMM_MEMDISP] = print_movsd_xmm_memdisp,
+	[INSN_MOVSD_XMM_MEMINDEX] = print_movsd_xmm_memindex,
+	[INSN_MOVSD_XMM_MEMLOCAL] = print_movsd_xmm_memlocal,
+	[INSN_MOVSD_XMM_XMM] = print_movsd_xmm_xmm,
+	[INSN_MOVSS_MEMBASE_XMM] = print_movss_membase_xmm,
+	[INSN_MOVSS_MEMDISP_XMM] = print_movss_memdisp_xmm,
+	[INSN_MOVSS_MEMINDEX_XMM] = print_movss_memindex_xmm,
+	[INSN_MOVSS_MEMLOCAL_XMM] = print_movss_memlocal_xmm,
+	[INSN_MOVSS_XMM_MEMBASE] = print_movss_xmm_membase,
+	[INSN_MOVSS_XMM_MEMDISP] = print_movss_xmm_memdisp,
+	[INSN_MOVSS_XMM_MEMINDEX] = print_movss_xmm_memindex,
+	[INSN_MOVSS_XMM_MEMLOCAL] = print_movss_xmm_memlocal,
+	[INSN_MOVSS_XMM_XMM] = print_movss_xmm_xmm,
+	[INSN_MOVSX_16_REG_REG] = print_movsx_16_reg_reg,
+	[INSN_MOVSX_8_REG_REG] = print_movsx_8_reg_reg,
+	[INSN_MOVZX_16_REG_REG] = print_movzx_16_reg_reg,
 	[INSN_MOV_IMM_MEMBASE] = print_mov_imm_membase,
 	[INSN_MOV_IMM_MEMLOCAL] = print_mov_imm_memlocal,
 	[INSN_MOV_IMM_REG] = print_mov_imm_reg,
 	[INSN_MOV_IMM_THREAD_LOCAL_MEMBASE] = print_mov_imm_tlmembase,
-	[INSN_MOV_MEMLOCAL_REG] = print_mov_memlocal_reg,
-	[INSN_MOV_MEMLOCAL_XMM] = print_mov_memlocal_xmm,
-	[INSN_MOV_64_MEMLOCAL_XMM] = print_mov_64_memlocal_xmm,
 	[INSN_MOV_MEMBASE_REG] = print_mov_membase_reg,
 	[INSN_MOV_MEMDISP_REG] = print_mov_memdisp_reg,
-	[INSN_MOV_MEMDISP_XMM] = print_mov_memdisp_xmm,
-	[INSN_MOV_64_MEMDISP_XMM] = print_mov_64_memdisp_xmm,
-	[INSN_MOV_MEMINDEX_XMM] = print_mov_memindex_xmm,
-	[INSN_MOV_64_MEMINDEX_XMM] = print_mov_64_memindex_xmm,
-	[INSN_MOV_REG_MEMDISP] = print_mov_reg_memdisp,
-	[INSN_MOV_THREAD_LOCAL_MEMDISP_REG] = print_mov_tlmemdisp_reg,
 	[INSN_MOV_MEMINDEX_REG] = print_mov_memindex_reg,
+	[INSN_MOV_MEMLOCAL_REG] = print_mov_memlocal_reg,
 	[INSN_MOV_REG_MEMBASE] = print_mov_reg_membase,
+	[INSN_MOV_REG_MEMDISP] = print_mov_reg_memdisp,
 	[INSN_MOV_REG_MEMINDEX] = print_mov_reg_memindex,
 	[INSN_MOV_REG_MEMLOCAL] = print_mov_reg_memlocal,
+	[INSN_MOV_REG_REG] = print_mov_reg_reg,
 	[INSN_MOV_REG_THREAD_LOCAL_MEMBASE] = print_mov_reg_tlmembase,
 	[INSN_MOV_REG_THREAD_LOCAL_MEMDISP] = print_mov_reg_tlmemdisp,
-	[INSN_MOV_XMM_MEMLOCAL] = print_mov_xmm_memlocal,
-	[INSN_MOV_64_XMM_MEMLOCAL] = print_mov_64_xmm_memlocal,
-	[INSN_MOV_REG_REG] = print_mov_reg_reg,
-	[INSN_MOV_XMM_MEMDISP] = print_mov_xmm_memdisp,
-	[INSN_MOV_64_XMM_MEMDISP] = print_mov_64_xmm_memdisp,
-	[INSN_MOV_XMM_MEMINDEX] = print_mov_xmm_memindex,
-	[INSN_MOV_64_XMM_MEMINDEX] = print_mov_64_xmm_memindex,
-	[INSN_MOV_XMM_XMM] = print_mov_xmm_xmm,
-	[INSN_MOV_64_XMM_XMM] = print_mov_64_xmm_xmm,
-	[INSN_MOVSX_8_REG_REG] = print_movsx_8_reg_reg,
-	[INSN_MOVSX_16_REG_REG] = print_movsx_16_reg_reg,
-	[INSN_MOVZX_16_REG_REG] = print_movzx_16_reg_reg,
+	[INSN_MOV_THREAD_LOCAL_MEMDISP_REG] = print_mov_tlmemdisp_reg,
+	[INSN_MULSD_MEMDISP_XMM] = print_fmul_64_memdisp_xmm,
+	[INSN_MULSD_XMM_XMM] = print_mulsd_xmm_xmm,
+	[INSN_MULSS_XMM_XMM] = print_mulss_xmm_xmm,
 	[INSN_MUL_MEMBASE_EAX] = print_mul_membase_eax,
 	[INSN_MUL_REG_EAX] = print_mul_reg_eax,
 	[INSN_MUL_REG_REG] = print_mul_reg_reg,
@@ -1052,11 +1075,12 @@ static print_insn_fn insn_printers[] = {
 	[INSN_OR_IMM_MEMBASE] = print_or_imm_membase,
 	[INSN_OR_MEMBASE_REG] = print_or_membase_reg,
 	[INSN_OR_REG_REG] = print_or_reg_reg,
-	[INSN_PUSH_IMM] = print_push_imm,
-	[INSN_PUSH_REG] = print_push_reg,
-	[INSN_PUSH_MEMLOCAL] = print_push_memlocal,
+	[INSN_PHI] = print_phi,
 	[INSN_POP_MEMLOCAL] = print_pop_memlocal,
 	[INSN_POP_REG] = print_pop_reg,
+	[INSN_PUSH_IMM] = print_push_imm,
+	[INSN_PUSH_MEMLOCAL] = print_push_memlocal,
+	[INSN_PUSH_REG] = print_push_reg,
 	[INSN_RET] = print_ret,
 	[INSN_SAR_IMM_REG] = print_sar_imm_reg,
 	[INSN_SAR_REG_REG] = print_sar_reg_reg,
@@ -1065,15 +1089,17 @@ static print_insn_fn insn_printers[] = {
 	[INSN_SBB_REG_REG] = print_sbb_reg_reg,
 	[INSN_SHL_REG_REG] = print_shl_reg_reg,
 	[INSN_SHR_REG_REG] = print_shr_reg_reg,
+	[INSN_SUBSD_XMM_XMM] = print_subsd_xmm_xmm,
+	[INSN_SUBSS_XMM_XMM] = print_subss_xmm_xmm,
 	[INSN_SUB_IMM_REG] = print_sub_imm_reg,
 	[INSN_SUB_MEMBASE_REG] = print_sub_membase_reg,
 	[INSN_SUB_REG_REG] = print_sub_reg_reg,
 	[INSN_TEST_IMM_MEMDISP] = print_test_imm_memdisp,
 	[INSN_TEST_MEMBASE_REG] = print_test_membase_reg,
+	[INSN_XORPD_XMM_XMM] = print_xor_64_xmm_reg_reg,
+	[INSN_XORPS_XMM_XMM] = print_xor_xmm_reg_reg,
 	[INSN_XOR_MEMBASE_REG] = print_xor_membase_reg,
 	[INSN_XOR_REG_REG] = print_xor_reg_reg,
-	[INSN_XOR_XMM_REG_REG] = print_xor_xmm_reg_reg,
-	[INSN_XOR_64_XMM_REG_REG] = print_xor_64_xmm_reg_reg,
 };
 
 int lir_print(struct insn *insn, struct string *str)
