@@ -68,6 +68,11 @@ int select_instructions(struct compilation_unit *cu);
 int compute_dfns(struct compilation_unit *cu);
 int compute_dom(struct compilation_unit *cu);
 int compute_dom_frontier(struct compilation_unit *cu);
+int lir_to_ssa(struct compilation_unit *cu);
+int ssa_to_lir(struct compilation_unit *cu);
+int dce(struct compilation_unit *cu);
+void imm_copy_propagation(struct compilation_unit *cu);
+void abc_removal(struct compilation_unit *cu);
 int allocate_registers(struct compilation_unit *cu);
 int insert_spill_reload_insns(struct compilation_unit *cu);
 int emit_machine_code(struct compilation_unit *);
@@ -112,6 +117,7 @@ extern regex_t method_trace_regex;
 extern bool opt_trace_gate;
 extern regex_t method_trace_gate_regex;
 
+extern bool opt_trace_ssa;
 extern bool opt_trace_cfg;
 extern bool opt_trace_tree_ir;
 extern bool opt_trace_lir;
@@ -125,6 +131,7 @@ extern bool opt_trace_invoke_verbose;
 extern bool opt_trace_exceptions;
 extern bool opt_trace_bytecode;
 extern bool opt_trace_compile;
+extern bool opt_print_compilation;
 
 extern bool opt_ssa_enable;
 extern bool running_on_valgrind;
@@ -137,6 +144,7 @@ static inline bool cu_matches_regex(struct compilation_unit *cu)
 	return method_matches_regex(cu->method);
 }
 
+void trace_ssa(struct compilation_unit *);
 void trace_magic_trampoline(struct compilation_unit *);
 void trace_method(struct compilation_unit *);
 void trace_cfg(struct compilation_unit *);
@@ -152,5 +160,7 @@ void trace_exception_unwind(struct jit_stack_frame *);
 void trace_exception_unwind_to_native(struct jit_stack_frame *);
 void trace_bytecode(struct vm_method *);
 void trace_return_value(struct vm_method *, unsigned long long);
+void print_method(struct vm_method *);
+void print_compilation(struct vm_method *);
 
 #endif
