@@ -1,6 +1,8 @@
 #ifndef JATO_EMIT_CODE_H
 #define JATO_EMIT_CODE_H
 
+#include "jit/stack-slot.h"
+
 struct compilation_unit;
 struct jit_trampoline;
 struct basic_block;
@@ -10,24 +12,7 @@ struct vm_object;
 struct vm_jni_env;
 struct vm_method;
 
-enum emitter_type {
-	NO_OPERANDS = 1,
-	SINGLE_OPERAND,
-	TWO_OPERANDS,
-	BRANCH,
-};
-
-struct emitter {
-	void *emit_fn;
-	enum emitter_type type;
-};
-
-extern struct emitter emitters[];
-
-#define DECL_EMITTER(_insn_type, _fn, _emitter_type) \
-	[_insn_type] = { .emit_fn = _fn, .type = _emitter_type }
-
-extern void emit_prolog(struct buffer *, unsigned long);
+extern void emit_prolog(struct buffer *, struct stack_frame *, unsigned long);
 extern void emit_trace_invoke(struct buffer *, struct compilation_unit *);
 extern void emit_epilog(struct buffer *);
 extern void emit_trampoline(struct compilation_unit *, void *, struct jit_trampoline *);
@@ -43,4 +28,7 @@ extern void backpatch_branch_target(struct buffer *buf, struct insn *insn,
 				    unsigned long target_offset);
 extern void emit_jni_trampoline(struct buffer *, struct vm_method *, void *);
 
+extern void *emit_ic_check(struct buffer *);
+extern void emit_ic_miss_handler(struct buffer *, void *, struct vm_method *);
+
 #endif /* JATO_EMIT_CODE_H */
