@@ -48,6 +48,17 @@
 
 #define convert_goto_w		convert_goto
 
+#define convert_areturn convert_xreturn
+#define convert_dreturn convert_xreturn
+#define convert_freturn convert_xreturn
+#define convert_ireturn convert_xreturn
+#define convert_lreturn convert_xreturn
+
+#define convert_dcmpg convert_xcmpg
+#define convert_fcmpg convert_xcmpg
+#define convert_dcmpl convert_xcmpl
+#define convert_fcmpl convert_xcmpl
+
 #define BYTECODE(opc, name, size, type) [opc] = convert_ ## name,
 static convert_fn_t converters[] = {
 #  include <vm/bytecode-def.h>
@@ -158,6 +169,7 @@ error_oom:
 int convert_instruction(struct parse_context *ctx)
 {
 	convert_fn_t convert;
+	int err;
 
 	ctx->opc = bytecode_read_u8(ctx->buffer);
 
@@ -168,7 +180,10 @@ int convert_instruction(struct parse_context *ctx)
 	if (!convert)
 		return warn("no converter for %d found", ctx->opc), -EINVAL;
 
-	int err = convert(ctx);
+	if ((ctx->opc >= OPC_IALOAD && ctx->opc <= OPC_SALOAD)
+		|| (ctx->opc >= OPC_IASTORE && ctx->opc <= OPC_SASTORE))
+		ctx->cu->flags |= CU_FLAG_ARRAY_OPC;
+	err = convert(ctx);
 
 	if (err)
 		warn("conversion error at PC=%lu", ctx->offset);
