@@ -59,6 +59,7 @@ static jint do_execute(uint8_t *code, unsigned long code_length)
 		.name		= "foo",
 		.class		= &vmc,
 		.code_attribute = (struct cafebabe_code_attribute) {
+			.max_locals	= 8,
 			.code		= code,
 			.code_length	= code_length,
 		},
@@ -100,6 +101,7 @@ static jdouble do_jdouble_execute(uint8_t *code, unsigned long code_length)
 		.name		= "foo",
 		.class		= &vmc,
 		.code_attribute = (struct cafebabe_code_attribute) {
+			.max_locals	= 4,
 			.code		= code,
 			.code_length	= code_length,
 		},
@@ -141,6 +143,7 @@ static jfloat do_jfloat_execute(uint8_t *code, unsigned long code_length)
 		.name		= "foo",
 		.class		= &vmc,
 		.code_attribute = (struct cafebabe_code_attribute) {
+			.max_locals	= 4,
 			.code		= code,
 			.code_length	= code_length,
 		},
@@ -182,6 +185,7 @@ static jobject do_jobject_execute(uint8_t *code, unsigned long code_length)
 		.name		= "foo",
 		.class		= &vmc,
 		.code_attribute = (struct cafebabe_code_attribute) {
+			.max_locals	= 4,
 			.code		= code,
 			.code_length	= code_length,
 		},
@@ -276,7 +280,7 @@ static void do_assert_float_equals(const char *function, const char *file, int l
 static void do_assert_object_equals(const char *function, const char *file, int line, jobject expected, jobject actual)
 {
 	if (expected != actual)
-		die("%s:%d::%s: Expected %d, but was: %d", file, line, function, expected, actual);
+		die("%s:%d::%s: Expected %p, but was: %p", file, line, function, expected, actual);
 
 	nr_assertions++;
 }
@@ -290,6 +294,7 @@ static void test_bipush(void)
 	assert_int_equals(16, execute(bytecode));
 }
 
+#ifndef CONFIG_ARM
 static void test_sipush(void)
 {
 	uint8_t bytecode[] = { OPC_SIPUSH, 0x01, 0x00, OPC_IRETURN };
@@ -355,6 +360,20 @@ static void test_if_icmpge(void)
 	assert_int_equals(1, execute(bytecode_3));
 }
 
+static void test_if_acmpeq(void)
+{
+	uint8_t bytecode_1[] = { OPC_ACONST_NULL, OPC_ACONST_NULL, OPC_IF_ACMPEQ, 0x00, 0x05, OPC_ICONST_1, OPC_IRETURN, OPC_ICONST_0, OPC_IRETURN };
+
+	assert_int_equals(0, execute(bytecode_1));
+}
+
+static void test_if_acmpne(void)
+{
+	uint8_t bytecode_1[] = { OPC_ACONST_NULL, OPC_ACONST_NULL, OPC_IF_ACMPNE, 0x00, 0x05, OPC_ICONST_1, OPC_IRETURN, OPC_ICONST_0, OPC_IRETURN };
+
+	assert_int_equals(1, execute(bytecode_1));
+}
+
 static void test_ifle(void)
 {
 	uint8_t bytecode_1[] = { OPC_ICONST_M1, OPC_IFLE, 0x00, 0x05, OPC_ICONST_2, OPC_IRETURN, OPC_ICONST_1, OPC_IRETURN };
@@ -483,10 +502,11 @@ static void test_ifnull(void)
 
 static void test_ifnonnull(void)
 {
-	uint8_t bytecode[] = { OPC_ACONST_NULL, OPC_IFNONNULL, 0x00, 0x00, OPC_ICONST_2, OPC_IRETURN, OPC_ICONST_1, OPC_IRETURN };
+	uint8_t bytecode[] = { OPC_ACONST_NULL, OPC_IFNONNULL, 0x00, 0x05, OPC_ICONST_2, OPC_IRETURN, OPC_ICONST_1, OPC_IRETURN };
 
 	assert_int_equals(2, execute(bytecode));
 }
+#endif
 
 static void test_ireturn(void)
 {
@@ -502,6 +522,7 @@ static void test_lreturn(void)
 	assert_long_equals(1, execute(bytecode));
 }
 
+#ifndef CONFIG_ARM
 static void test_freturn(void)
 {
 	uint8_t bytecode[] = { OPC_FCONST_1, OPC_FRETURN };
@@ -522,7 +543,17 @@ static void test_areturn(void)
 
 	assert_object_equals(NULL, jobject_run(bytecode));
 }
+#endif
 
+static void test_return(void)
+{
+	uint8_t bytecode[] = { OPC_RETURN };
+
+	/* OPC_RETURN doesn't have a return value */
+	jobject_run(bytecode);
+}
+
+#ifndef CONFIG_ARM
 static void test_lxor(void)
 {
 	uint8_t bytecode[] = { OPC_LCONST_1, OPC_LCONST_1, OPC_LXOR, OPC_LRETURN };
@@ -690,6 +721,7 @@ static void test_imul(void)
 
 	assert_int_equals(6, execute(bytecode));
 }
+#endif
 
 static void test_lsub(void)
 {
@@ -705,6 +737,7 @@ static void test_isub(void)
 	assert_int_equals(1, execute(bytecode));
 }
 
+#ifndef CONFIG_ARM
 static void test_dadd(void)
 {
 	uint8_t bytecode[] = { OPC_DCONST_1, OPC_DCONST_1, OPC_DADD, OPC_DRETURN };
@@ -760,6 +793,7 @@ static void test_fdiv(void)
 
 	assert_float_equals(2.0, jfloat_run(bytecode));
 }
+#endif
 
 static void test_ladd(void)
 {
@@ -775,6 +809,7 @@ static void test_iadd(void)
 	assert_int_equals(3, execute(bytecode));
 }
 
+#ifndef CONFIG_ARM
 static void test_swap(void)
 {
 	uint8_t bytecode_1[] = { OPC_ICONST_1, OPC_ICONST_2, OPC_SWAP, OPC_IRETURN };
@@ -836,6 +871,7 @@ static void test_fconst_0(void)
 
 	assert_float_equals(0.0, jfloat_run(bytecode));
 }
+#endif
 
 static void test_lconst_1(void)
 {
@@ -907,6 +943,120 @@ static void test_iconst_m1(void)
 	assert_int_equals(-1, execute(bytecode));
 }
 
+static void test_istore_0(void)
+{
+	uint8_t bytecode[] = { OPC_ICONST_1, OPC_ISTORE_0, OPC_ILOAD_0, OPC_IRETURN};
+
+	assert_int_equals(1, execute(bytecode));
+}
+
+static void test_istore_1(void)
+{
+	uint8_t bytecode[] = { OPC_ICONST_1, OPC_ISTORE_1, OPC_ILOAD_1, OPC_IRETURN};
+
+	assert_int_equals(1, execute(bytecode));
+}
+
+static void test_istore_2(void)
+{
+	uint8_t bytecode[] = { OPC_ICONST_1, OPC_ISTORE_2, OPC_ILOAD_2, OPC_IRETURN};
+
+	assert_int_equals(1, execute(bytecode));
+}
+
+static void test_istore_3(void)
+{
+	uint8_t bytecode[] = { OPC_ICONST_1, OPC_ISTORE_3, OPC_ILOAD_3, OPC_IRETURN};
+
+	assert_int_equals(1, execute(bytecode));
+}
+
+static void test_lstore_0(void)
+{
+	uint8_t bytecode[] = { OPC_LCONST_1, OPC_LSTORE_0, OPC_LLOAD_0, OPC_LRETURN};
+
+	assert_long_equals(1, jlong_run(bytecode));
+}
+
+static void test_lstore_1(void)
+{
+	uint8_t bytecode[] = { OPC_LCONST_1, OPC_LSTORE_1, OPC_LLOAD_1, OPC_LRETURN};
+
+	assert_long_equals(1, jlong_run(bytecode));
+}
+
+static void test_lstore_2(void)
+{
+	uint8_t bytecode[] = { OPC_LCONST_1, OPC_LSTORE_2, OPC_LLOAD_2, OPC_LRETURN};
+
+	assert_long_equals(1, jlong_run(bytecode));
+}
+
+static void test_lstore_3(void)
+{
+	uint8_t bytecode[] = { OPC_LCONST_1, OPC_LSTORE_3, OPC_LLOAD_3, OPC_LRETURN};
+
+	assert_long_equals(1, jlong_run(bytecode));
+}
+
+static void test_fstore_0(void)
+{
+	uint8_t bytecode[] = { OPC_FCONST_1, OPC_FSTORE_0, OPC_FLOAD_0, OPC_FRETURN};
+
+	assert_float_equals(1, jfloat_run(bytecode));
+}
+
+static void test_fstore_1(void)
+{
+	uint8_t bytecode[] = { OPC_FCONST_1, OPC_FSTORE_1, OPC_FLOAD_1, OPC_FRETURN};
+
+	assert_float_equals(1, jfloat_run(bytecode));
+}
+
+static void test_fstore_2(void)
+{
+	uint8_t bytecode[] = { OPC_FCONST_1, OPC_FSTORE_2, OPC_FLOAD_2, OPC_FRETURN};
+
+	assert_float_equals(1, jfloat_run(bytecode));
+}
+
+static void test_fstore_3(void)
+{
+	uint8_t bytecode[] = { OPC_FCONST_1, OPC_FSTORE_3, OPC_FLOAD_3, OPC_FRETURN};
+
+	assert_float_equals(1, jfloat_run(bytecode));
+}
+
+static void test_dstore_0(void)
+{
+	uint8_t bytecode[] = { OPC_DCONST_1, OPC_DSTORE_0, OPC_DLOAD_0, OPC_DRETURN};
+
+	assert_double_equals(1, jdouble_run(bytecode));
+}
+
+static void test_dstore_1(void)
+{
+	uint8_t bytecode[] = { OPC_DCONST_1, OPC_DSTORE_1, OPC_DLOAD_1, OPC_DRETURN};
+
+	assert_double_equals(1, jdouble_run(bytecode));
+}
+
+static void test_dstore_2(void)
+{
+	uint8_t bytecode[] = { OPC_DCONST_1, OPC_DSTORE_2, OPC_DLOAD_2, OPC_DRETURN};
+
+	assert_double_equals(1, jdouble_run(bytecode));
+}
+
+static void test_dstore_3(void)
+{
+	uint8_t bytecode[] = { OPC_DCONST_1, OPC_DSTORE_3, OPC_DLOAD_3, OPC_DRETURN};
+
+	assert_double_equals(1, jdouble_run(bytecode));
+}
+
+#ifndef CONFIG_ARM
+
 static void test_i2f(void)
 {
 	uint8_t bytecode[] = { OPC_ICONST_1, OPC_I2F, OPC_FRETURN };
@@ -984,6 +1134,20 @@ static void test_d2f(void)
 	assert_float_equals(1, jfloat_run(bytecode));
 }
 
+static void test_i2b(void)
+{
+	uint8_t bytecode[] = { OPC_BIPUSH, 0xFF, OPC_I2B, OPC_IRETURN };
+
+	assert_int_equals(-1, execute(bytecode));
+}
+
+static void test_i2c(void)
+{
+	uint8_t bytecode[] = { OPC_SIPUSH, 0xFF, 0xFF, OPC_I2C, OPC_IRETURN };
+
+	assert_int_equals(0xffff, execute(bytecode));
+}
+
 static void test_i2s(void)
 {
 	uint8_t bytecode[] = { OPC_SIPUSH, 0xFF, 0xFF, OPC_I2S, OPC_IRETURN };
@@ -997,6 +1161,7 @@ static void test_i2l(void)
 
 	assert_long_equals(1, execute(bytecode));
 }
+#endif
 
 static void run_tests(void)
 {
@@ -1011,12 +1176,15 @@ static void run_tests(void)
 	test_iconst_5();
 	test_lconst_0();
 	test_lconst_1();
+#ifndef CONFIG_ARM
 	test_fconst_0();
 	test_fconst_1();
 	test_fconst_2();
 	test_dconst_0();
 	test_dconst_1();
+#endif
 	test_bipush();
+#ifndef CONFIG_ARM
 	test_sipush();
 	/* test_ldc(); */
 	/* test_ldc_w(); */
@@ -1059,22 +1227,24 @@ static void run_tests(void)
 	/* test_fstore(); */
 	/* test_dstore(); */
 	/* test_astore(); */
-	/* test_istore_0(); */
-	/* test_istore_1(); */
-	/* test_istore_2(); */
-	/* test_istore_3(); */
-	/* test_lstore_0(); */
-	/* test_lstore_1(); */
-	/* test_lstore_2(); */
-	/* test_lstore_3(); */
-	/* test_fstore_0(); */
-	/* test_fstore_1(); */
-	/* test_fstore_2(); */
-	/* test_fstore_3(); */
-	/* test_dstore_0(); */
-	/* test_dstore_1(); */
-	/* test_dstore_2(); */
-	/* test_dstore_3(); */
+#endif
+	test_istore_0();
+	test_istore_1();
+	test_istore_2();
+	test_istore_3();
+	test_lstore_0();
+	test_lstore_1();
+	test_lstore_2();
+	test_lstore_3();
+	test_fstore_0();
+	test_fstore_1();
+	test_fstore_2();
+	test_fstore_3();
+	test_dstore_0();
+	test_dstore_1();
+	test_dstore_2();
+	test_dstore_3();
+#ifndef CONFIG_ARM
 	/* test_astore_0(); */
 	/* test_astore_1(); */
 	/* test_astore_2(); */
@@ -1096,12 +1266,16 @@ static void run_tests(void)
 	/* test_dup2_x1(); */
 	/* test_dup2_x2(); */
 	test_swap();
+#endif
 	test_iadd();
 	test_ladd();
+#ifndef CONFIG_ARM
 	test_fadd();
 	test_dadd();
+#endif
 	test_isub();
 	test_lsub();
+#ifndef CONFIG_ARM
 	test_fsub();
 	test_dsub();
 	test_imul();
@@ -1145,8 +1319,8 @@ static void run_tests(void)
 	test_d2i();
 	test_d2l();
 	test_d2f();
-	/* test_i2b(); */
-	/* test_i2c(); */
+	test_i2b();
+	test_i2c();
 	test_i2s();
 	test_lcmp();
 	test_fcmpl();
@@ -1165,19 +1339,23 @@ static void run_tests(void)
 	test_if_icmpge();
 	test_if_icmpgt();
 	test_if_icmple();
-	/* test_if_acmpeq(); */
-	/* test_if_acmpne(); */
+	test_if_acmpeq();
+	test_if_acmpne();
 	test_goto();
 	/* test_jsr(); */
 	/* test_ret(); */
 	/* test_tableswitch(); */
 	/* test_lookupswitch(); */
+#endif
 	test_ireturn();
 	test_lreturn();
+#ifndef CONFIG_ARM
 	test_freturn();
 	test_dreturn();
 	test_areturn();
-	/* test_return(); */
+#endif
+	test_return();
+#ifndef CONFIG_ARM
 	/* test_getstatic(); */
 	/* test_putstatic(); */
 	/* test_getfield(); */
@@ -1201,14 +1379,13 @@ static void run_tests(void)
 	test_ifnonnull();
 	test_goto_w();
 	/* test_jsr_w(); */
+#endif
 }
 
 int main(int argc, char *argv[])
 {
 	dont_gc = true;
 
-	opt_trace_machine_code = true;
-
 	preload_finished = true;
 
 	init();
