@@ -27,6 +27,9 @@
 
 #include "vm/class.h"
 
+#include "cafebabe/enclosing_method_attribute.h"
+#include "cafebabe/inner_classes_attribute.h"
+#include "cafebabe/annotations_attribute.h"
 #include "cafebabe/constant_pool.h"
 #include "cafebabe/method_info.h"
 #include "cafebabe/field_info.h"
@@ -36,9 +39,11 @@
 #include "jit/exception.h"
 #include "jit/compiler.h"
 #include "jit/vtable.h"
+#include "jit/cu-mapping.h"
 
 #include "vm/fault-inject.h"
 #include "vm/classloader.h"
+#include "vm/annotation.h"
 #include "vm/gc.h"
 #include "vm/preload.h"
 #include "vm/errors.h"
@@ -50,6 +55,7 @@
 #include "vm/field.h"
 #include "vm/die.h"
 #include "vm/vm.h"
+#include "vm/trace.h"
 
 #include "lib/string.h"
 #include "lib/array.h"
@@ -60,6 +66,30 @@
 #include <stdio.h>
 #include <ctype.h>
 
+bool opt_trace_vtable;
+
+static void
+trace_vtable(struct vm_class *vmc)
+{
+	struct vtable *vtable = &(vmc->vtable);
+	struct compilation_unit *cu;
+	struct vm_method *vmm;
+
+	trace_printf("trace vtable: %s\n", vmc->name);
+
+	for (uint16_t i = 0; i < vmc->vtable_size; ++i) {
+		cu = jit_lookup_cu((long unsigned int)vtable->native_ptr[i]);
+		if (cu) {
+			vmm = cu->method;
+			trace_printf("  vtable[%d] = %s.%s%s\n", i, vmm->class->name,
+				     vmm->name, vmm->type);
+		}
+	}
+
+	trace_flush();
+}
+
+
 static void
 setup_vtable(struct vm_class *vmc)
 {
@@ -75,6 +105,7 @@ setup_vtable(struct vm_class *vmc)
 		super_vtable = &super->vtable;
 	} else {
 		super_vtable_size = 0;
+		super_vtable = NULL;
 	}
 
 	vtable_size = 0;
@@ -112,24 +143,49 @@ setup_vtable(struct vm_class *vmc)
 				    vmm->virtual_index,
 				    vm_method_call_ptr(vmm));
 	}
+
+	if (opt_trace_vtable)
+		trace_vtable(vmc);
+}
+
+/*
+ * Set the .object member of struct vm_class to point to the object
+ * of type java.lang.Class for this class.
+ */
+int vm_class_setup_object(struct vm_class *vmc)
+{
+	vmc->object = vm_object_alloc(vm_java_lang_Class);
+	if (!vmc->object) {
+		throw_oom_error();
+		return -1;
+	}
+
+	field_set_object(vmc->object, vm_java_lang_Class_vmdata,
+			 (struct vm_object *)vmc);
+	return 0;
 }
 
 static int vm_class_link_common(struct vm_class *vmc)
 {
 	int err;
 
+	compile_lock_init(&vmc->cl, true);
+
 	err = pthread_mutex_init(&vmc->mutex, NULL);
 	if (err)
 		return -err;
 
-	err = vm_monitor_init(&vmc->monitor);
-	if (err)
-		return -err;
+	if (preload_finished)
+		return vm_class_setup_object(vmc);
 
-	vmc->object = NULL;
-	vmc->classloader = NULL;
+	/* If classes and fields were not preloaded yet then we can't
+	 * allocate object for this class. Set .object to a dummy object
+	 * for class synchronization to work. This will be replaced
+	 * after preloading is finished. */
+	static struct vm_object dummy_object;
+	vmc->object = &dummy_object;
 
-	return 0;
+	return vm_preload_add_class_fixup(vmc);
 }
 
 /*
@@ -239,6 +295,7 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 {
 	const struct cafebabe_constant_info_class *constant_class;
 	const struct cafebabe_constant_info_utf8 *name;
+	unsigned int nr_inner_classes;
 
 	vmc->class = class;
 	vmc->kind = VM_CLASS_KIND_REGULAR;
@@ -271,7 +328,7 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 		char *super_name_str = strndup((char *) super_name->bytes, super_name->length);
 
 		/* XXX: Circularity check */
-		vmc->super = classloader_load(NULL, super_name_str);
+		vmc->super = classloader_load(vmc->classloader, super_name_str);
 		free(super_name_str);
 
 		if (!vmc->super)
@@ -303,7 +360,7 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 		if (!c_name)
 			goto error_free_interfaces;
 
-		struct vm_class *vmi = classloader_load(NULL, c_name);
+		struct vm_class *vmi = classloader_load(vmc->classloader, c_name);
 		free(c_name);
 		if (!vmi)
 			goto error_free_interfaces;
@@ -311,11 +368,12 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 		vmc->interfaces[i] = vmi;
 	}
 
+	vmc->nr_fields = class->fields_count;
 	vmc->fields = vm_alloc(sizeof(*vmc->fields) * class->fields_count);
 	if (!vmc->fields)
 		goto error_free_interfaces;
 
-	for (uint16_t i = 0; i < class->fields_count; ++i) {
+	for (uint16_t i = 0; i < vmc->nr_fields; ++i) {
 		struct vm_field *vmf = &vmc->fields[i];
 
 		if (vm_field_init(vmf, vmc, i))
@@ -351,8 +409,6 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 		for (unsigned int j = 0; j < 2; ++j) {
 			struct field_bucket *bucket = &field_buckets[j][i];
 
-			// TODO: free fields explicitly after they are no longer
-			// used.
 			bucket->fields = vm_alloc(bucket->nr * sizeof(*bucket->fields));
 			bucket->nr = 0;
 		}
@@ -377,7 +433,7 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 	if (!vmc->static_values)
 		goto error_free_buckets;
 
-	for (uint16_t i = 0; i < class->fields_count; ++i) {
+	for (uint16_t i = 0; i < vmc->nr_fields; ++i) {
 		struct vm_field *vmf = &vmc->fields[i];
 
 		if (vm_field_is_static(vmf)) {
@@ -460,17 +516,81 @@ int vm_class_link(struct vm_class *vmc, const struct cafebabe_class *class)
 
 	INIT_LIST_HEAD(&vmc->static_fixup_site_list);
 
+	struct cafebabe_inner_classes_attribute inner_classes_attribute;
+
+	memset(&inner_classes_attribute, 0, sizeof(inner_classes_attribute));
+	if (cafebabe_read_inner_classes_attribute(class, &class->attributes, &inner_classes_attribute))
+		return -1;
+
+	nr_inner_classes = 0;
+	for (unsigned int i = 0; i < inner_classes_attribute.number_of_classes; i++) {
+		struct cafebabe_inner_class *inner = &inner_classes_attribute.inner_classes[i];
+
+		if (class->this_class == inner->outer_class_info_index)
+			nr_inner_classes++;
+	}
+
+	vmc->inner_classes = vm_alloc(sizeof(*vmc->inner_classes) * nr_inner_classes);
+	if (!vmc->inner_classes)
+		goto error_free_methods;
+
+	for (unsigned int i = 0; i < inner_classes_attribute.number_of_classes; i++) {
+		struct cafebabe_inner_class *inner = &inner_classes_attribute.inner_classes[i];
+
+		if (class->this_class == inner->inner_class_info_index) {
+			vmc->declaring_class = vmc->enclosing_class = vm_class_resolve_class(vmc, inner->outer_class_info_index);
+
+			vmc->inner_class_access_flags	= inner->inner_class_access_flags;
+		} else if (class->this_class == inner->outer_class_info_index) {
+			vmc->inner_classes[vmc->nr_inner_classes++] = inner->inner_class_info_index;
+		}
+	}
+
+	struct cafebabe_annotations_attribute annotations_attribute;
+
+	if (cafebabe_read_annotations_attribute(class, &class->attributes, &annotations_attribute))
+		goto error_free_inner_classes;
+
+	vmc->annotations = vm_alloc(sizeof(struct vm_annotation *) * annotations_attribute.num_annotations);
+	if (!vmc->annotations)
+		goto error_free_inner_classes;
+
+	for (unsigned int i = 0; i < annotations_attribute.num_annotations; i++) {
+		struct cafebabe_annotation *annotation = &annotations_attribute.annotations[i];
+		struct vm_annotation *vma;
+
+		vma = vm_annotation_parse(vmc, annotation);
+		if (!vma)
+			goto error_free_annotations;
+
+		vmc->annotations[vmc->nr_annotations++] = vma;
+	}
+
+	cafebabe_annotations_attribute_deinit(&annotations_attribute);
+
+	if (!cafebabe_read_enclosing_method_attribute(class, &class->attributes, &vmc->enclosing_method_attribute))
+		vmc->enclosing_class = vm_class_resolve_class(vmc, vmc->enclosing_method_attribute.class_index);
+
 	vmc->state = VM_CLASS_LINKED;
 	return 0;
 
+error_free_annotations:
+	for (unsigned int i = 0; i < vmc->nr_annotations; i++) {
+		struct vm_annotation *vma = vmc->annotations[i];
+
+		vm_annotation_free(vma);
+	}
+	vm_free(vmc->annotations);
 error_free_methods:
-	free(vmc->methods);
+	vm_free(vmc->methods);
+error_free_inner_classes:
+	vm_free(vmc->inner_classes);
 error_free_static_values:
-	free(vmc->static_values);
+	vm_free(vmc->static_values);
 error_free_buckets:
 	free_buckets(2, VM_TYPE_MAX, field_buckets);
 error_free_fields:
-	free(vmc->fields);
+	vm_free(vmc->fields);
 error_free_interfaces:
 	free(vmc->interfaces);
 error_free_name:
@@ -566,61 +686,31 @@ static bool vm_class_check_class_init_fault(struct vm_class *vmc,
 
 int vm_class_ensure_object(struct vm_class *vmc)
 {
-	vm_monitor_lock(&vmc->monitor);
-
-	if (vmc->object)
-		goto out;
-
-	/*
-	 * Set the .object member of struct vm_class to point to
-	 * the object (of type java.lang.Class) for this class.
-	 */
-	vmc->object = vm_object_alloc(vm_java_lang_Class);
-	if (!vmc->object) {
-		NOT_IMPLEMENTED;
-		vm_monitor_unlock(&vmc->monitor);
-		return -1;
-	}
-
-	field_set_object(vmc->object, vm_java_lang_Class_vmdata,
-		(struct vm_object *)vmc);
-
- out:
-	vm_monitor_unlock(&vmc->monitor);
+	/* Before preload is finished .object points to a common
+	 * dummy object. */
+	assert(preload_finished);
 	return 0;
 }
 
 int vm_class_init(struct vm_class *vmc)
 {
 	struct vm_object *exception;
+	enum compile_lock_status status;
 
-	vm_monitor_lock(&vmc->monitor);
+	status = compile_lock_enter(&vmc->cl);
+	if (status == STATUS_COMPILED_OK || status == STATUS_REENTER)
+		return 0;
 
-	if (vmc->state == VM_CLASS_INITIALIZING) {
-		/* XXX: we need to break recursion. */
-		if (vmc->initializing_thread == vm_thread_self())
-			goto out_unlock;
-
-		while (vmc->state == VM_CLASS_INITIALIZING)
-			vm_monitor_wait(&vmc->monitor);
-	}
-
-	if (vmc->state == VM_CLASS_INITIALIZED)
-		goto out_unlock;
-
-	if (vmc->state == VM_CLASS_ERRONEOUS) {
+	if (status == STATUS_COMPILED_ERRONOUS) {
 		signal_new_exception(vm_java_lang_NoClassDefFoundError,
 				     vmc->name);
-		vm_monitor_unlock(&vmc->monitor);
 		return -1;
 	}
 
+	vm_object_lock(vmc->object);
 	assert(vmc->state == VM_CLASS_LINKED);
-
 	vmc->state = VM_CLASS_INITIALIZING;
-	vmc->initializing_thread = vm_thread_self();
-
-	vm_monitor_unlock(&vmc->monitor);
+	vm_object_unlock(vmc->object);
 
 	/* Fault injection, for testing purposes */
 	if (vm_fault_enabled(VM_FAULT_CLASS_INIT)) {
@@ -661,18 +751,16 @@ int vm_class_init(struct vm_class *vmc)
 		}
 	}
 
-	vm_monitor_lock(&vmc->monitor);
+	vm_object_lock(vmc->object);
 	vmc->state = VM_CLASS_INITIALIZED;
-	vm_monitor_notify_all(&vmc->monitor);
-	vm_monitor_unlock(&vmc->monitor);
+	vm_object_notify_all(vmc->object);
+	vm_object_unlock(vmc->object);
 
-	return 0;
-
- out_unlock:
-	vm_monitor_unlock(&vmc->monitor);
+	compile_lock_leave(&vmc->cl, STATUS_COMPILED_OK);
 	return 0;
 
  error:
+	compile_lock_leave(&vmc->cl, STATUS_COMPILED_ERRONOUS);
 	exception = exception_occurred();
 
 	if (!vm_object_is_instance_of(exception, vm_java_lang_Error)) {
@@ -682,10 +770,10 @@ int vm_class_init(struct vm_class *vmc)
 			vmc->name);
 	}
 
-	vm_monitor_lock(&vmc->monitor);
+	vm_object_lock(vmc->object);
 	vmc->state = VM_CLASS_ERRONEOUS;
-	vm_monitor_notify_all(&vmc->monitor);
-	vm_monitor_unlock(&vmc->monitor);
+	vm_object_notify_all(vmc->object);
+	vm_object_unlock(vmc->object);
 
 	return -1;
 }
@@ -693,12 +781,9 @@ int vm_class_init(struct vm_class *vmc)
 struct vm_class *vm_class_resolve_class(const struct vm_class *vmc, uint16_t i)
 {
 	const struct cafebabe_constant_info_class *constant_class;
-	if (cafebabe_class_constant_get_class(vmc->class,
-		i, &constant_class))
-	{
-		NOT_IMPLEMENTED;
+
+	if (cafebabe_class_constant_get_class(vmc->class, i, &constant_class))
 		return NULL;
-	}
 
 	const struct cafebabe_constant_info_utf8 *class_name;
 	if (cafebabe_class_constant_get_utf8(vmc->class,
@@ -719,9 +804,10 @@ struct vm_class *vm_class_resolve_class(const struct vm_class *vmc, uint16_t i)
 		= classloader_load(vmc->classloader, class_name_str);
 	if (!class) {
 		warn("failed to load class %s", class_name_str);
-		return NULL;
+		goto out;
 	}
-
+out:
+	free(class_name_str);
 	return class;
 }
 
@@ -838,33 +924,15 @@ vm_class_resolve_field_recursive(const struct vm_class *vmc, uint16_t i)
 	return result;
 }
 
-int vm_class_resolve_method(const struct vm_class *vmc, uint16_t i,
-	struct vm_class **r_vmc, char **r_name, char **r_type)
+static int vm_class_resolve_name_and_type(const struct vm_class *vmc, uint16_t index, char **r_name, char **r_type)
 {
-	const struct cafebabe_constant_info_method_ref *method;
-	if (cafebabe_class_constant_get_method_ref(vmc->class, i, &method)) {
-		NOT_IMPLEMENTED;
-		return -1;
-	}
-
-	struct vm_class *class = vm_class_resolve_class(vmc,
-		method->class_index);
-	if (!class) {
-		NOT_IMPLEMENTED;
-		return -1;
-	}
-
 	const struct cafebabe_constant_info_name_and_type *name_and_type;
-	if (cafebabe_class_constant_get_name_and_type(vmc->class,
-		method->name_and_type_index, &name_and_type))
-	{
-		NOT_IMPLEMENTED;
+
+	if (cafebabe_class_constant_get_name_and_type(vmc->class, index, &name_and_type))
 		return -1;
-	}
 
 	const struct cafebabe_constant_info_utf8 *name;
-	if (cafebabe_class_constant_get_utf8(vmc->class,
-		name_and_type->name_index, &name))
+	if (cafebabe_class_constant_get_utf8(vmc->class, name_and_type->name_index, &name))
 	{
 		NOT_IMPLEMENTED;
 		return -1;
@@ -890,9 +958,32 @@ int vm_class_resolve_method(const struct vm_class *vmc, uint16_t i,
 		return -1;
 	}
 
-	*r_vmc = class;
 	*r_name = name_str;
 	*r_type = type_str;
+
+	return 0;
+}
+
+int vm_class_resolve_method(const struct vm_class *vmc, uint16_t i,
+	struct vm_class **r_vmc, char **r_name, char **r_type)
+{
+	const struct cafebabe_constant_info_method_ref *method;
+	if (cafebabe_class_constant_get_method_ref(vmc->class, i, &method)) {
+		NOT_IMPLEMENTED;
+		return -1;
+	}
+
+	struct vm_class *class = vm_class_resolve_class(vmc, method->class_index);
+	if (!class) {
+		NOT_IMPLEMENTED;
+		return -1;
+	}
+
+	if (vm_class_resolve_name_and_type(vmc, method->name_and_type_index, r_name, r_type))
+		return -1;
+
+	*r_vmc = class;
+
 	return 0;
 }
 
@@ -1003,13 +1094,58 @@ int vm_class_resolve_interface_method(const struct vm_class *vmc, uint16_t i,
 	return 0;
 }
 
-struct vm_method *
-vm_class_resolve_method_recursive(const struct vm_class *vmc, uint16_t i)
+static struct vm_method *
+missing_method(struct vm_class *vmc, char *name, char *type, uint16_t access_flags)
 {
+	struct cafebabe_method_info *method;
+	struct vm_method *vmm;
+
+	method		= vm_alloc(sizeof *method);
+	if (!method)
+		return NULL;
+
+	method->access_flags	= access_flags;
+
+	vmm		= vm_alloc(sizeof *vmm);
+	if (!vmm)
+		goto error_free_method;
+
+	vmm->class	= vmc;
+
+	vmm->name	= strdup(name);
+	if (!vmm->name)
+		goto error_free_vmm;
+
+	vmm->type	= strdup(type);
+	if (!vmm->type)
+		goto error_free_name;
+
+	vmm->method	= method;
+
+	if (vm_method_do_init(vmm))
+		goto error_free_type;
+
+	return vmm;
+
+error_free_type:
+	free(vmm->type);
+error_free_name:
+	free(vmm->name);
+error_free_vmm:
+	free(vmm);
+error_free_method:
+	free(method);
+
+	return NULL;
+}
+
+struct vm_method *
+vm_class_resolve_method_recursive(const struct vm_class *vmc, uint16_t i, uint16_t access_flags)
+{
+	struct vm_method *result;
 	struct vm_class *class;
 	char *name;
 	char *type;
-	struct vm_method *result;
 
 	if (vm_class_resolve_method(vmc, i, &class, &name, &type)) {
 		NOT_IMPLEMENTED;
@@ -1017,9 +1153,12 @@ vm_class_resolve_method_recursive(const struct vm_class *vmc, uint16_t i)
 	}
 
 	result = vm_class_get_method_recursive(class, name, type);
+	if (!result)
+		result = missing_method(class, name, type, access_flags);
 
 	free(name);
 	free(type);
+
 	return result;
 }
 
@@ -1054,8 +1193,28 @@ static bool is_numeric(const char *s)
 	return true;
 }
 
+struct vm_method *vm_class_get_enclosing_method(const struct vm_class *vmc)
+{
+	struct vm_method *result;
+	char *name;
+	char *type;
+
+	if (!vmc->enclosing_class)
+		return NULL;
+
+	if (vm_class_resolve_name_and_type(vmc, vmc->enclosing_method_attribute.method_index, &name, &type))
+		return NULL;
+
+	result = vm_class_get_method_recursive(vmc->enclosing_class, name, type);
+
+	free(name);
+	free(type);
+
+	return result;
+}
+
 /* See Section 15.9.5 ("Anonymous Class Declarations") of the JLS for details.  */
-bool vm_class_is_anonymous(struct vm_class *vmc)
+bool vm_class_is_anonymous(const struct vm_class *vmc)
 {
 	if (!vm_class_is_regular_class(vmc))
 		return false;
@@ -1074,33 +1233,79 @@ bool vm_class_is_anonymous(struct vm_class *vmc)
 	return is_numeric(separator + 1);
 }
 
-/* Reference: http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Class.html#isAssignableFrom(java.lang.Class) */
-bool vm_class_is_assignable_from(const struct vm_class *vmc, const struct vm_class *from)
+static void supertype_cache_update(struct vm_class *vmc, const struct vm_class *super)
+{
+	unsigned int ndx = vmc->supertype_cache_ndx++ % SUPERTYPE_CACHE_SIZE;
+
+	vmc->supertype_cache[ndx]       = super;
+}
+
+static bool vm_class_is_subclass_of(const struct vm_class *vmc, const struct vm_class *from)
+{
+	struct vm_class *super = from->super;
+
+	while (super) {
+		if (vmc == super)
+			return true;
+
+		super = super->super;
+	}
+
+	return false;
+}
+
+static bool vm_class_is_assignable_from_nocache(const struct vm_class *vmc, const struct vm_class *from);
+
+static bool vm_class_is_instance_of_array(const struct vm_class *vmc, const struct vm_class *from)
+{
+	if (!vm_class_is_array_class(from))
+		return false;
+
+	struct vm_class *vmc_el = vm_class_get_array_element_class(vmc);
+
+	struct vm_class *from_el = vm_class_get_array_element_class(vmc);
+
+	return vm_class_is_assignable_from_nocache(vmc_el, from_el);
+}
+
+static bool vm_class_implements(const struct vm_class *vmc, const struct vm_class *from)
+{
+	for (unsigned int i = 0; i < from->nr_interfaces; ++i) {
+		if (vmc == from->interfaces[i] || vm_class_implements(vmc, from->interfaces[i]))
+			return true;
+	}
+
+	if (from->super)
+		return vm_class_implements(vmc, from->super);
+
+	return false;
+}
+
+static bool vm_class_is_assignable_from_nocache(const struct vm_class *vmc, const struct vm_class *from)
 {
 	if (vmc == from)
 		return true;
 
-	if (vm_class_is_array_class(vmc)) {
-		if (!vm_class_is_array_class(from))
-			return false;
+	if (vm_class_is_interface(vmc))
+		return vm_class_implements(vmc, from);
 
-		const struct vm_class *vmc_el
-			= vm_class_get_array_element_class(vmc);
-		const struct vm_class *from_el
-			= vm_class_get_array_element_class(vmc);
+	if (vm_class_is_array_class(vmc))
+		return vm_class_is_instance_of_array(vmc, from);
 
-		return vm_class_is_assignable_from(vmc_el, from_el);
-	}
+	return vm_class_is_subclass_of(vmc, from);
+}
 
-	if (from->super && vm_class_is_assignable_from(vmc, from->super))
-		return true;
+/* Reference: http://download.oracle.com/javase/1.5.0/docs/api/java/lang/Class.html#isAssignableFrom(java.lang.Class) */
+bool vm_class_is_assignable_from_slow(struct vm_class *vmc, const struct vm_class *from)
+{
+	bool ret;
 
-	for (unsigned int i = 0; i < from->nr_interfaces; ++i) {
-		if (vm_class_is_assignable_from(vmc, from->interfaces[i]))
-			return true;
-	}
+	ret = vm_class_is_assignable_from_nocache(vmc, from);
 
-	return false;
+	if (ret)
+		supertype_cache_update(vmc, from);
+
+	return ret;
 }
 
 char *vm_class_get_array_element_class_name(const char *class_name)
@@ -1147,7 +1352,7 @@ struct vm_class *vm_class_get_array_class(struct vm_class *element_class)
 	struct vm_class *result;
 	char *name;
 
-	if (!asprintf(&name, "[%s", element_class->name))
+	if (!asprintf(&name, "[L%s;", element_class->name))
 		return throw_oom_error();
 
 	result = classloader_load(element_class->classloader, name);
@@ -1156,7 +1361,8 @@ struct vm_class *vm_class_get_array_class(struct vm_class *element_class)
 }
 
 struct vm_class *
-vm_class_define(const char *name, uint8_t *data, unsigned long len)
+vm_class_define(struct vm_object *classloader, const char *name,
+		uint8_t *data, unsigned long len)
 {
 	struct cafebabe_stream stream;
 	struct cafebabe_class *class;
@@ -1176,6 +1382,8 @@ vm_class_define(const char *name, uint8_t *data, unsigned long len)
 	if (!result)
 		return throw_oom_error();
 
+	result->classloader = classloader;
+
 	if (vm_class_link(result, class))
 		return NULL;
 
