@@ -67,8 +67,6 @@ pthread_spinlock_t gc_spinlock;
 static int nr_in_safepoint;
 static int nr_threads;
 
-static __thread sig_atomic_t in_safepoint;
-
 static pthread_t gc_thread_id;
 
 unsigned long max_heap_size	= 128 * 1024 * 1024;	/* 128 MB */
@@ -123,13 +121,15 @@ static bool do_exit_safepoint(void)
 {
 	bool ret = false;
 
-	assert(in_safepoint);
+	assert(vm_get_exec_env()->in_safepoint);
 
-	in_safepoint = false;
+	vm_get_exec_env()->in_safepoint = false;
 
 	if (pthread_spin_lock(&gc_spinlock) != 0)
 		die("pthread_spin_lock");
 
+	assert(nr_in_safepoint > 0);
+
 	if (--nr_in_safepoint == 0)
 		ret = true;
 
@@ -147,13 +147,15 @@ static void exit_safepoint(void)
 
 static void enter_safepoint(void)
 {
-	assert(!in_safepoint);
+	assert(!vm_get_exec_env()->in_safepoint);
 
-	in_safepoint = true;
+	vm_get_exec_env()->in_safepoint = true;
 
 	if (pthread_spin_lock(&gc_spinlock) != 0)
 		die("pthread_spin_lock");
 
+	assert(nr_in_safepoint < nr_threads);
+
 	if (++nr_in_safepoint == nr_threads)
 		resume_thread(gc_thread_id);
 
@@ -208,12 +210,12 @@ void suspend_handler(int sig, siginfo_t *si, void *ctx)
 		 * Fresh threads might return NULL from vm_thread_self().
 		 */
 		if (self)
-			vm_thread_set_state(self, THREAD_STATE_CONSISTENT);
+			vm_thread_set_state(self, VM_THREAD_STATE_CONSISTENT);
 
 		save_signal_registers(&thread_register_state, &uc->uc_mcontext);
 		gc_safepoint(&thread_register_state);
 	} else {
-		vm_thread_set_state(self, THREAD_STATE_CONSISTENT);
+		vm_thread_set_state(self, VM_THREAD_STATE_CONSISTENT);
 
 		enter_safepoint();
 
@@ -289,7 +291,7 @@ static void gc_suspend_rest(void)
 	vm_thread_for_each(thread) {
 		assert(thread->posix_id != pthread_self());
 
-		if (vm_thread_get_state(thread) == THREAD_STATE_INCONSISTENT) {
+		if (vm_thread_get_state(thread) == VM_THREAD_STATE_INCONSISTENT) {
 			resume_thread(thread->posix_id);
 			nr_restarted++;
 		}
@@ -369,7 +371,10 @@ static void *gc_thread(void *arg)
 	 * by default to avoid races with sigwait().
 	 */
 	sigemptyset(&sigset);
-	sigaddset(&sigset, SIGUSR2);
+
+	if(sigaddset(&sigset, SIGUSR2) != 0)
+		die("sigaddset");
+
 	pthread_sigmask(SIG_BLOCK, &sigset, NULL);
 
 	for (;;) {
@@ -418,6 +423,9 @@ static void *do_gc_alloc(size_t size)
 
 	p	= malloc(size);
 
+	if (p)
+		memset(p, 0, size);
+
 	return p;
 }
 
@@ -444,20 +452,46 @@ static void do_vm_free(void *p)
 	free(p);
 }
 
-static int do_gc_register_finalizer(struct vm_object *object,
-				    finalizer_fn finalizer)
+static int do_gc_register_finalizer(struct vm_object *object, finalizer_fn finalizer)
 {
-	NOT_IMPLEMENTED;
 	return 0;
 }
 
+static void do_gc_setup_signals(void)
+{
+	struct sigaction sa;
+	sigset_t sigset;
+
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags     = SA_RESTART | SA_SIGINFO;
+
+	sa.sa_sigaction = suspend_handler;
+	sigaction(SIGUSR1, &sa, NULL);
+
+	sa.sa_sigaction = wakeup_handler;
+	sigaction(SIGUSR2, &sa, NULL);
+
+	/*
+	 * SIGUSR2 is used to resume threads. Make sure the signal is blocked
+	 * by default to avoid races with sigwait().
+	 */
+	sigemptyset(&sigset);
+
+	if (sigaddset(&sigset, SIGUSR2) != 0)
+		die("sigaddset");
+
+	pthread_sigmask(SIG_BLOCK, &sigset, NULL);
+}
+
 static void gc_setup(void)
 {
 	gc_ops		= (struct gc_operations) {
 		.gc_alloc		= do_gc_alloc,
+		.gc_alloc_noscan	= do_gc_alloc,
 		.vm_alloc		= do_vm_alloc,
 		.vm_free		= do_vm_free,
-		.gc_register_finalizer	= do_gc_register_finalizer
+		.gc_register_finalizer	= do_gc_register_finalizer,
+		.gc_setup_signals	= do_gc_setup_signals,
 	};
 
 	if (pthread_spin_init(&gc_spinlock, PTHREAD_PROCESS_SHARED) != 0)
