@@ -52,27 +52,28 @@
 
 #include "runtime/stack-walker.h"
 #include "runtime/runtime.h"
-#include "runtime/unsafe.h"
-#include "runtime/class.h"
-#include "runtime/classloader.h"
 
 #include "jit/compiler.h"
 #include "jit/cu-mapping.h"
 #include "jit/gdb.h"
 #include "jit/exception.h"
+#include "jit/inline-cache.h"
 #include "jit/perf-map.h"
 #include "jit/text.h"
 
+#include "lib/string.h"
 #include "lib/parse.h"
 #include "lib/list.h"
 
 #include "vm/fault-inject.h"
+#include "vm/verifier.h"
 #include "vm/classloader.h"
 #include "vm/stack-trace.h"
 #include "vm/reflection.h"
 #include "vm/natives.h"
 #include "vm/preload.h"
 #include "vm/version.h"
+#include "vm/interp.h"
 #include "vm/itable.h"
 #include "vm/method.h"
 #include "vm/object.h"
@@ -84,31 +85,54 @@
 #include "vm/thread.h"
 #include "vm/class.h"
 #include "vm/call.h"
+#include "vm/utf8.h"
 #include "vm/jar.h"
 #include "vm/jni.h"
 #include "vm/gc.h"
 #include "vm/vm.h"
+#include "vm/java-version.h"
 
 #include "arch/init.h"
 
+#include "runtime/gnu_java_lang_management_VMThreadMXBeanImpl.h"
+#include "runtime/java_lang_reflect_VMMethod.h"
+#include "runtime/java_lang_reflect_VMField.h"
+#include "runtime/java_lang_VMClassLoader.h"
+#include "runtime/java_lang_VMString.h"
+#include "runtime/java_lang_VMSystem.h"
+#include "runtime/java_lang_VMThread.h"
+#include "runtime/java_lang_VMClass.h"
+#include "runtime/sun_misc_Unsafe.h"
+
+static const char *bootclasspath_append;
+
 static bool dump_maps;
 static bool perf_enabled;
-static char *exe_name;
+static char *program_name;
 
 /* Arguments passed to the main class.  */
 static unsigned int nr_java_args;
 static char **java_args;
 
+static bool use_system_classloader = true;
+
 /*
  * Enable SSA optimizations.
  */
 bool opt_ssa_enable;
 
+static bool opt_interp_only;
+
 /*
  * Enable JIT workarounds for valgrind.
  */
 bool running_on_valgrind;
 
+static void vm_atexit(void)
+{
+	classloader_destroy();
+}
+
 static void __attribute__((noreturn)) vm_exit(int status)
 {
 	clear_exception();
@@ -125,6 +149,9 @@ static void vm_properties_set_property(struct vm_object *p,
 	struct vm_object *(*trampoline)(struct vm_object *,
 					struct vm_object *, struct vm_object *);
 
+	assert(key != NULL);
+	assert(value != NULL);
+
 	trampoline
 		= vm_method_trampoline_ptr(vm_java_util_Properties_setProperty);
 
@@ -225,8 +252,6 @@ struct system_property {
 	const char *value;
 };
 
-#define JAVA_VERSION		"1.5.0"
-
 static struct system_property system_properties[] = {
 	{ "java.vm.name",			"jato"				},
 	{ "java.vm.vendor",			"Pekka Enberg"			},
@@ -316,78 +341,6 @@ static void native_vmruntime_println(struct vm_object *message)
 	free(cstr);
 }
 
-static void
-native_vmsystem_arraycopy(struct vm_object *src, int src_start,
-			  struct vm_object *dest, int dest_start, int len)
-{
-	const struct vm_class *src_elem_class;
-	const struct vm_class *dest_elem_class;
-	enum vm_type elem_type;
-	int elem_size;
-
-	if (!src || !dest || !src->class || !dest->class) {
-		signal_new_exception(vm_java_lang_NullPointerException, NULL);
-		return;
-	}
-
-	if (!vm_class_is_array_class(src->class) ||
-	    !vm_class_is_array_class(dest->class)) {
-		signal_new_exception(vm_java_lang_ArrayStoreException, NULL);
-		return;
-	}
-
-	src_elem_class = vm_class_get_array_element_class(src->class);
-	dest_elem_class = vm_class_get_array_element_class(dest->class);
-	if (!src_elem_class || !dest_elem_class) {
-		signal_new_exception(vm_java_lang_NullPointerException, NULL);
-		return;
-	}
-
-	elem_type = vm_class_get_storage_vmtype(src_elem_class);
-	if (elem_type != vm_class_get_storage_vmtype(dest_elem_class)) {
-		NOT_IMPLEMENTED;
-		return;
-	}
-
-	if (len < 0 ||
-	    src_start < 0 || src_start + len > vm_array_length(src) ||
-	    dest_start < 0 || dest_start + len > vm_array_length(dest)) {
-		signal_new_exception(
-			vm_java_lang_ArrayIndexOutOfBoundsException, NULL);
-		return;
-	}
-
-	elem_size = vmtype_get_size(elem_type);
-	memmove(vm_array_elems(dest) + dest_start * elem_size,
-		vm_array_elems(src) + src_start * elem_size,
-		len * elem_size);
-
-	return;
-}
-
-static int32_t hash_ptr_to_int32(void *p)
-{
-#ifdef CONFIG_64_BIT
-	int64_t key = (int64_t) p;
-
-	key = (~key) + (key << 18);
-	key = key ^ (key >> 31);
-	key = key * 21;
-	key = key ^ (key >> 11);
-	key = key + (key << 6);
-	key = key ^ (key >> 22);
-
-	return key;
-#else
-	return (int32_t) p;
-#endif
-}
-
-static int32_t native_vmsystem_identityhashcode(struct vm_object *obj)
-{
-	return hash_ptr_to_int32(obj);
-}
-
 static struct vm_object *
 native_vmobject_clone(struct vm_object *object)
 {
@@ -443,48 +396,6 @@ native_vm_throw_null_pointer_exception(void)
 	signal_new_exception(vm_java_lang_NullPointerException, NULL);
 }
 
-static struct vm_object *native_vmthread_current_thread(void)
-{
-	return field_get_object(vm_get_exec_env()->thread->vmthread,
-				vm_java_lang_VMThread_thread);
-}
-
-static void native_vmthread_start(struct vm_object *vmthread, jlong stacksize)
-{
-	vm_thread_start(vmthread);
-}
-
-static void native_vmthread_yield(void)
-{
-	vm_thread_yield();
-}
-
-static jint native_vmthread_interrupted(void)
-{
-	return vm_thread_interrupted(vm_thread_self());
-}
-
-static jint native_vmthread_isinterrupted(struct vm_object *vmthread)
-{
-	struct vm_thread *thread;
-
-	thread = vm_thread_from_vmthread(vmthread);
-	return vm_thread_is_interrupted(thread);
-}
-
-static void native_vmthread_setpriority(struct vm_object *vmthread, jint ptiority)
-{
-	/* TODO: implement */
-}
-
-static void native_vmthread_interrupt(struct vm_object *vmthread)
-{
-	struct vm_thread *thread;
-
-	thread = (struct vm_thread *)field_get_object(vmthread, vm_java_lang_VMThread_vmdata);
-	vm_thread_interrupt(thread);
-}
-
 static void native_vmobject_notify(struct vm_object *obj)
 {
 	vm_object_notify(obj);
@@ -509,19 +420,15 @@ static void native_vmobject_wait(struct vm_object *object, jlong ms, jint ns)
 		vm_object_timed_wait(object, ms, ns);
 }
 
-static struct vm_object *native_vmstring_intern(struct vm_object *str)
-{
-	return vm_string_intern(str);
-}
-
 static jint native_atomiclong_vm_supports_cs8(void)
 {
 	return false;
 }
 
 static struct vm_native natives[] = {
+	DEFINE_NATIVE("gnu/java/lang/management/VMThreadMXBeanImpl", "getThreadInfoForId", gnu_java_lang_management_VMThreadMXBeanImpl_getThreadInfoForId),
 	DEFINE_NATIVE("gnu/classpath/VMStackWalker", "getClassContext", native_vmstackwalker_getclasscontext),
-	DEFINE_NATIVE("gnu/classpath/VMStackWalker", "getClassLoader", native_vmclass_getclassloader),
+	DEFINE_NATIVE("gnu/classpath/VMStackWalker", "getClassLoader", java_lang_VMClass_getClassLoader),
 	DEFINE_NATIVE("gnu/classpath/VMSystemProperties", "preInit", native_vmsystemproperties_preinit),
 	DEFINE_NATIVE("jato/internal/VM", "disableFault", native_vm_disable_fault),
 	DEFINE_NATIVE("jato/internal/VM", "enableFault", native_vm_enable_fault),
@@ -530,28 +437,33 @@ static struct vm_native natives[] = {
 	DEFINE_NATIVE("jato/internal/VM", "throwNullPointerException", native_vm_throw_null_pointer_exception),
 	DEFINE_NATIVE("java/lang/reflect/VMArray", "createObjectArray", native_vmarray_createobjectarray),
 	DEFINE_NATIVE("java/io/VMFile", "isDirectory", native_vmfile_is_directory),
-	DEFINE_NATIVE("java/lang/VMClass", "forName", native_vmclass_forname),
-	DEFINE_NATIVE("java/lang/VMClass", "getClassLoader", native_vmclass_getclassloader),
-	DEFINE_NATIVE("java/lang/VMClass", "getComponentType", native_vmclass_getcomponenttype),
-	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredConstructors", native_vmclass_get_declared_constructors),
-	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredFields", native_vmclass_get_declared_fields),
-	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredMethods", native_vmclass_get_declared_methods),
-	DEFINE_NATIVE("java/lang/VMClass", "getDeclaringClass", native_vmclass_get_declaring_class),
-	DEFINE_NATIVE("java/lang/VMClass", "getInterfaces", native_vmclass_get_interfaces),
-	DEFINE_NATIVE("java/lang/VMClass", "getModifiers", native_vmclass_getmodifiers),
-	DEFINE_NATIVE("java/lang/VMClass", "getName", native_vmclass_getname),
-	DEFINE_NATIVE("java/lang/VMClass", "getSuperclass", native_vmclass_get_superclass),
-	DEFINE_NATIVE("java/lang/VMClass", "isAnonymousClass", native_vmclass_is_anonymous_class),
-	DEFINE_NATIVE("java/lang/VMClass", "isArray", native_vmclass_isarray),
-	DEFINE_NATIVE("java/lang/VMClass", "isAssignableFrom", native_vmclass_is_assignable_from),
-	DEFINE_NATIVE("java/lang/VMClass", "isInstance", native_vmclass_isinstance),
-	DEFINE_NATIVE("java/lang/VMClass", "isInterface", native_vmclass_isinterface),
-	DEFINE_NATIVE("java/lang/VMClass", "isPrimitive", native_vmclass_isprimitive),
-	DEFINE_NATIVE("java/lang/VMClassLoader", "defineClass", native_vmclassloader_defineclass),
-	DEFINE_NATIVE("java/lang/VMClassLoader", "findLoadedClass", native_vmclassloader_findloadedclass),
-	DEFINE_NATIVE("java/lang/VMClassLoader", "getPrimitiveClass", native_vmclassloader_getprimitiveclass),
-	DEFINE_NATIVE("java/lang/VMClassLoader", "loadClass", native_vmclassloader_loadclass),
-	DEFINE_NATIVE("java/lang/VMClassLoader", "resolveClass", native_vmclassloader_resolveclass),
+	DEFINE_NATIVE("java/lang/VMClass", "forName", java_lang_VMClass_forName),
+	DEFINE_NATIVE("java/lang/VMClass", "getClassLoader", java_lang_VMClass_getClassLoader),
+	DEFINE_NATIVE("java/lang/VMClass", "getComponentType", java_lang_VMClass_getComponentType),
+	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredAnnotations", java_lang_VMClass_getDeclaredAnnotations),
+	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredClasses", java_lang_VMClass_getDeclaredClasses),
+	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredConstructors", java_lang_VMClass_getDeclaredConstructors),
+	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredFields", java_lang_VMClass_getDeclaredFields),
+	DEFINE_NATIVE("java/lang/VMClass", "getDeclaredMethods", java_lang_VMClass_getDeclaredMethods),
+	DEFINE_NATIVE("java/lang/VMClass", "getDeclaringClass", java_lang_VMClass_getDeclaringClass),
+	DEFINE_NATIVE("java/lang/VMClass", "getEnclosingClass", java_lang_VMClass_getEnclosingClass),
+	DEFINE_NATIVE("java/lang/VMClass", "getInterfaces", java_lang_VMClass_getInterfaces),
+	DEFINE_NATIVE("java/lang/VMClass", "getModifiers", java_lang_VMClass_getModifiers),
+	DEFINE_NATIVE("java/lang/VMClass", "getName", java_lang_VMClass_getName),
+	DEFINE_NATIVE("java/lang/VMClass", "getSuperclass", java_lang_VMClass_getSuperclass),
+	DEFINE_NATIVE("java/lang/VMClass", "isAnonymousClass", java_lang_VMClass_isAnonymousClass),
+	DEFINE_NATIVE("java/lang/VMClass", "isArray", java_lang_VMClass_isArray),
+	DEFINE_NATIVE("java/lang/VMClass", "isAssignableFrom", java_lang_VMClass_isAssignableFrom),
+	DEFINE_NATIVE("java/lang/VMClass", "isInstance", java_lang_VMClass_isInstance),
+	DEFINE_NATIVE("java/lang/VMClass", "isInterface", java_lang_VMClass_isInterface),
+	DEFINE_NATIVE("java/lang/VMClass", "isLocalClass", java_lang_VMClass_isLocalClass),
+	DEFINE_NATIVE("java/lang/VMClass", "isMemberClass", java_lang_VMClass_isMemberClass),
+	DEFINE_NATIVE("java/lang/VMClass", "isPrimitive", java_lang_VMClass_isPrimitive),
+	DEFINE_NATIVE("java/lang/VMClassLoader", "defineClass", java_lang_VMClassLoader_defineClass),
+	DEFINE_NATIVE("java/lang/VMClassLoader", "findLoadedClass", java_lang_VMClassLoader_findLoadedClass),
+	DEFINE_NATIVE("java/lang/VMClassLoader", "getPrimitiveClass", java_lang_VMClassLoader_getPrimitiveClass),
+	DEFINE_NATIVE("java/lang/VMClassLoader", "loadClass", java_lang_VMClassLoader_loadClass),
+	DEFINE_NATIVE("java/lang/VMClassLoader", "resolveClass", java_lang_VMClassLoader_resolveClass),
 	DEFINE_NATIVE("java/lang/VMObject", "clone", native_vmobject_clone),
 	DEFINE_NATIVE("java/lang/VMObject", "getClass", native_vmobject_getclass),
 	DEFINE_NATIVE("java/lang/VMObject", "notify", native_vmobject_notify),
@@ -566,16 +478,19 @@ static struct vm_native natives[] = {
 	DEFINE_NATIVE("java/lang/VMRuntime", "mapLibraryName", native_vmruntime_maplibraryname),
 	DEFINE_NATIVE("java/lang/VMRuntime", "nativeLoad", native_vmruntime_native_load),
 	DEFINE_NATIVE("java/lang/VMRuntime", "runFinalizationForExit", native_vmruntime_run_finalization_for_exit),
-	DEFINE_NATIVE("java/lang/VMString", "intern", native_vmstring_intern),
-	DEFINE_NATIVE("java/lang/VMSystem", "arraycopy", native_vmsystem_arraycopy),
-	DEFINE_NATIVE("java/lang/VMSystem", "identityHashCode", native_vmsystem_identityhashcode),
-	DEFINE_NATIVE("java/lang/VMThread", "currentThread", native_vmthread_current_thread),
-	DEFINE_NATIVE("java/lang/VMThread", "interrupt", native_vmthread_interrupt),
-	DEFINE_NATIVE("java/lang/VMThread", "interrupted", native_vmthread_interrupted),
-	DEFINE_NATIVE("java/lang/VMThread", "isInterrupted", native_vmthread_isinterrupted),
-	DEFINE_NATIVE("java/lang/VMThread", "nativeSetPriority", native_vmthread_setpriority),
-	DEFINE_NATIVE("java/lang/VMThread", "start", native_vmthread_start),
-	DEFINE_NATIVE("java/lang/VMThread", "yield", native_vmthread_yield),
+	DEFINE_NATIVE("java/lang/VMRuntime", "runFinalization", java_lang_VMRuntime_runFinalization),
+	DEFINE_NATIVE("java/lang/VMRuntime", "traceInstructions", native_vmruntime_trace_instructions),
+	DEFINE_NATIVE("java/lang/VMRuntime", "traceMethodCalls", native_vmruntime_trace_method_calls),
+	DEFINE_NATIVE("java/lang/VMString", "intern", java_lang_VMString_intern),
+	DEFINE_NATIVE("java/lang/VMSystem", "arraycopy", java_lang_VMSystem_arraycopy),
+	DEFINE_NATIVE("java/lang/VMSystem", "identityHashCode", java_lang_VMSystem_identityHashCode),
+	DEFINE_NATIVE("java/lang/VMThread", "currentThread", java_lang_VMThread_currentThread),
+	DEFINE_NATIVE("java/lang/VMThread", "interrupt", java_lang_VMThread_interrupt),
+	DEFINE_NATIVE("java/lang/VMThread", "interrupted", java_lang_VMThread_interrupted),
+	DEFINE_NATIVE("java/lang/VMThread", "isInterrupted", java_lang_VMThread_isInterrupted),
+	DEFINE_NATIVE("java/lang/VMThread", "nativeSetPriority", java_lang_VMThread_setPriority),
+	DEFINE_NATIVE("java/lang/VMThread", "start", java_lang_VMThread_start),
+	DEFINE_NATIVE("java/lang/VMThread", "yield", java_lang_VMThread_yield),
 	DEFINE_NATIVE("java/lang/VMThrowable", "fillInStackTrace", native_vmthrowable_fill_in_stack_trace),
 	DEFINE_NATIVE("java/lang/VMThrowable", "getStackTrace", native_vmthrowable_get_stack_trace),
 	DEFINE_NATIVE("java/lang/reflect/Constructor", "constructNative", native_constructor_construct_native),
@@ -586,22 +501,41 @@ static struct vm_native natives[] = {
 	DEFINE_NATIVE("java/lang/reflect/VMConstructor", "getParameterTypes", native_constructor_get_parameter_types),
 	DEFINE_NATIVE("java/lang/reflect/VMConstructor", "getModifiersInternal", native_constructor_get_modifiers_internal),
 	DEFINE_NATIVE("java/lang/reflect/VMConstructor", "getExceptionTypes", native_vmconstructor_get_exception_types),
-	DEFINE_NATIVE("java/lang/reflect/Field", "get", native_field_get),
-	DEFINE_NATIVE("java/lang/reflect/Field", "getInt", native_field_get_int),
-	DEFINE_NATIVE("java/lang/reflect/Field", "getLong", native_field_get_long),
-	DEFINE_NATIVE("java/lang/reflect/Field", "getModifiersInternal", native_field_get_modifiers_internal),
-	DEFINE_NATIVE("java/lang/reflect/Field", "getType", native_field_gettype),
-	DEFINE_NATIVE("java/lang/reflect/Field", "set", native_field_set),
-	DEFINE_NATIVE("java/lang/reflect/VMField", "get", native_field_get),
-	DEFINE_NATIVE("java/lang/reflect/VMField", "getLong", native_field_get_long),
-	DEFINE_NATIVE("java/lang/reflect/VMField", "getInt", native_field_get_int),
-	DEFINE_NATIVE("java/lang/reflect/VMField", "getModifiersInternal", native_field_get_modifiers_internal),
-	DEFINE_NATIVE("java/lang/reflect/VMField", "getType", native_field_gettype),
-	DEFINE_NATIVE("java/lang/reflect/VMField", "set", native_field_set),
+	DEFINE_NATIVE("java/lang/reflect/Field", "get", java_lang_reflect_VMField_get),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getInt", java_lang_reflect_VMField_getInt),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getLong", java_lang_reflect_VMField_getLong),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getShort", java_lang_reflect_VMField_getShort),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getFloat", java_lang_reflect_VMField_getFloat),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getDouble", java_lang_reflect_VMField_getDouble),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getChar", java_lang_reflect_VMField_getChar),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getByte", java_lang_reflect_VMField_getByte),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getBoolean", java_lang_reflect_VMField_getBoolean),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getModifiersInternal", java_lang_reflect_VMField_getModifiersInternal),
+	DEFINE_NATIVE("java/lang/reflect/Field", "getType", java_lang_reflect_VMField_getType),
+	DEFINE_NATIVE("java/lang/reflect/Field", "set", java_lang_reflect_VMField_set),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "get", java_lang_reflect_VMField_get),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getLong", java_lang_reflect_VMField_getLong),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getInt", java_lang_reflect_VMField_getInt),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getShort", java_lang_reflect_VMField_getShort),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getFloat", java_lang_reflect_VMField_getFloat),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getDouble", java_lang_reflect_VMField_getDouble),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getChar", java_lang_reflect_VMField_getChar),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getByte", java_lang_reflect_VMField_getByte),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getBoolean", java_lang_reflect_VMField_getBoolean),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getModifiersInternal", java_lang_reflect_VMField_getModifiersInternal),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getType", java_lang_reflect_VMField_getType),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getAnnotation", java_lang_reflect_VMField_getAnnotation),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "getDeclaredAnnotations", java_lang_reflect_VMField_getDeclaredAnnotations),
+	DEFINE_NATIVE("java/lang/reflect/VMField", "set", java_lang_reflect_VMField_set),
 	DEFINE_NATIVE("java/lang/reflect/Method", "getModifiersInternal", native_method_get_modifiers_internal),
 	DEFINE_NATIVE("java/lang/reflect/Method", "getParameterTypes", native_method_get_parameter_types),
 	DEFINE_NATIVE("java/lang/reflect/Method", "getReturnType", native_method_getreturntype),
 	DEFINE_NATIVE("java/lang/reflect/Method", "invokeNative", native_method_invokenative),
+	DEFINE_NATIVE("java/lang/reflect/Method", "getExceptionTypes", native_method_get_exception_types),
+	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getAnnotation", java_lang_reflect_VMMethod_getAnnotation),
+	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getDeclaredAnnotations", java_lang_reflect_VMMethod_getDeclaredAnnotations),
+	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getParameterAnnotations", java_lang_reflect_VMMethod_getParameterAnnotations),
+	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getDefaultValue", native_method_get_default_value),
 	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getExceptionTypes", native_method_get_exception_types),
 	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getModifiersInternal", native_method_get_modifiers_internal),
 	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getParameterTypes", native_method_get_parameter_types),
@@ -609,12 +543,21 @@ static struct vm_native natives[] = {
 	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getReturnType", native_method_getreturntype),
 	DEFINE_NATIVE("java/lang/reflect/VMMethod", "getExceptionTypes", native_method_get_exception_types),
 	DEFINE_NATIVE("java/util/concurrent/atomic/AtomicLong", "VMSupportsCS8", native_atomiclong_vm_supports_cs8),
+	DEFINE_NATIVE("sun/misc/Unsafe", "arrayBaseOffset", sun_misc_Unsafe_arrayBaseOffset),
+	DEFINE_NATIVE("sun/misc/Unsafe", "arrayIndexScale", sun_misc_Unsafe_arrayIndexScale),
 	DEFINE_NATIVE("sun/misc/Unsafe", "compareAndSwapInt", native_unsafe_compare_and_swap_int),
 	DEFINE_NATIVE("sun/misc/Unsafe", "compareAndSwapLong", native_unsafe_compare_and_swap_long),
 	DEFINE_NATIVE("sun/misc/Unsafe", "compareAndSwapObject", native_unsafe_compare_and_swap_object),
-	DEFINE_NATIVE("sun/misc/Unsafe", "objectFieldOffset", native_unsafe_object_field_offset),
-	DEFINE_NATIVE("sun/misc/Unsafe", "putObject", native_unsafe_put_object),
+	DEFINE_NATIVE("sun/misc/Unsafe", "getIntVolatile", sun_misc_Unsafe_getIntVolatile),
+	DEFINE_NATIVE("sun/misc/Unsafe", "getLongVolatile", sun_misc_Unsafe_getLongVolatile),
+	DEFINE_NATIVE("sun/misc/Unsafe", "getObjectVolatile", sun_misc_Unsafe_getObjectVolatile),
+	DEFINE_NATIVE("sun/misc/Unsafe", "objectFieldOffset", sun_misc_Unsafe_objectFieldOffset),
 	DEFINE_NATIVE("sun/misc/Unsafe", "park", native_unsafe_park),
+	DEFINE_NATIVE("sun/misc/Unsafe", "putIntVolatile", sun_misc_Unsafe_putIntVolatile),
+	DEFINE_NATIVE("sun/misc/Unsafe", "putLong", sun_misc_Unsafe_putLong),
+	DEFINE_NATIVE("sun/misc/Unsafe", "putLongVolatile", sun_misc_Unsafe_putLongVolatile),
+	DEFINE_NATIVE("sun/misc/Unsafe", "putObject", sun_misc_Unsafe_putObject),
+	DEFINE_NATIVE("sun/misc/Unsafe", "putObjectVolatile", sun_misc_Unsafe_putObjectVolatile),
 	DEFINE_NATIVE("sun/misc/Unsafe", "unpark", native_unsafe_unpark),
 };
 
@@ -626,9 +569,29 @@ static void jit_init_natives(void)
 		vm_register_native(&natives[i]);
 }
 
+#define USAGE_TEXT									\
+	"Usage: %s [-options] class [arg1 arg2 ...]\n"					\
+	"                 (to run a class file)\n"					\
+	"   or  %s [-options] -jar jarfile [arg1 arg2 ...]\n"				\
+	"                 (to run a standalone jar file)\n"				\
+	"where options include:\n"							\
+	"  -client	   compatibility (ignored)\n"					\
+	"  -server	   compatibility (ignored)\n"					\
+	"\n"										\
+	"  -cp		   <jar/zip files and directories separated by :>\n"		\
+	"  -classpath	   <jar/zip files and directories separated by :>\n"		\
+	"		   locations where to find application classes\n"		\
+	"  -D<name>=<value> set a system property\n"					\
+	"  -verbose[:gc]\n"								\
+	"		   :gc print out results of garbage collection\n"		\
+	"  -version	   print out version number and copyright information\n"	\
+	"\n"										\
+	"  -Xint           operate in interpreter-only mode\n"				\
+	"  -XX:+PrintCompilation Print a message when a method is compiled\n"
+
 static void usage(FILE *f, int retval)
 {
-	fprintf(f, "usage: %s [options] class\n", exe_name);
+	fprintf(f, USAGE_TEXT, program_name, program_name);
 	exit(retval);
 }
 
@@ -644,26 +607,45 @@ static void handle_help(void)
 	usage(stdout, EXIT_SUCCESS);
 }
 
+static void handle_client(void)
+{
+	/* Ignore */
+}
+
+static void handle_server(void)
+{
+	/* Ignore */
+}
+
 static void handle_classpath(const char *arg)
 {
 	system_property_append_path("java.class.path", arg);
 }
 
-static void handle_bootclasspath(const char *arg)
+static void bootclasspath_parse_and_append(const void *arg)
 {
 	static const char delim[] = ":;";
 
-	add_system_property_const("java.boot.class.path", arg);
-
 	char *cp = strdup(arg);
 
 	char *path = strtok(cp, delim);
 	while (path) {
 		classloader_add_to_classpath(path);
+		system_property_append_path("java.boot.class.path", path);
 		path = strtok(NULL, delim);
 	}
 }
 
+static void handle_bootclasspath(const char *arg)
+{
+	bootclasspath_parse_and_append(arg);
+}
+
+static void handle_bootclasspath_append(const char *arg)
+{
+	bootclasspath_append = arg;
+}
+
 enum operation {
 	OPERATION_MAIN_CLASS,
 	OPERATION_JAR_FILE,
@@ -694,7 +676,7 @@ static void handle_jar(const char *arg)
 		exit(EXIT_FAILURE);
 	}
 
-	classname = strdup(main_class);
+	classname = dots_to_slash(main_class);
 	if (!classname)
 		die("out of memory");
 
@@ -709,6 +691,11 @@ static void handle_nogc(void)
 	dont_gc		= 1;
 }
 
+static void handle_no_system_classloader(void)
+{
+	use_system_classloader = false;
+}
+
 static void handle_newgc(void)
 {
 	newgc_enabled	= true;
@@ -729,6 +716,16 @@ static void handle_ssa(void)
 	opt_ssa_enable = true;
 }
 
+static void handle_no_ic(void)
+{
+	opt_ic_enabled  = false;
+}
+
+static void handle_int(void)
+{
+	opt_interp_only  = true;
+}
+
 static void regex_compile(regex_t *regex, const char *arg)
 {
 	int err = regcomp(regex, arg, REG_EXTENDED | REG_NOSUB);
@@ -768,6 +765,11 @@ static void handle_trace_bytecode_offset(void)
 	opt_trace_bytecode_offset = true;
 }
 
+static void handle_trace_verifier(void)
+{
+	opt_trace_verifier = true;
+}
+
 static void handle_trace_classloader(void)
 {
 	opt_trace_classloader = true;
@@ -801,6 +803,7 @@ static void handle_trace_itable(void)
 
 static void handle_trace_jit(void)
 {
+	opt_trace_ssa = true;
 	opt_trace_cfg = true;
 	opt_trace_tree_ir = true;
 	opt_trace_lir = true;
@@ -823,6 +826,11 @@ static void handle_trace_trampoline(void)
 	opt_trace_magic_trampoline = true;
 }
 
+static void handle_trace_vtable(void)
+{
+	opt_trace_vtable = true;
+}
+
 static void handle_define(const char *arg)
 {
 	char *str, *ptr, *key, *value;
@@ -862,11 +870,21 @@ static void handle_max_heap_size(const char *arg)
 	max_heap_size = parse_long(arg);
 
 	if (!max_heap_size) {
-		fprintf(stderr, "%s: unparseable heap size '%s'\n", exe_name, arg);
+		fprintf(stderr, "%s: unparseable heap size '%s'\n", program_name, arg);
 		usage(stderr, EXIT_FAILURE);
 	}
 }
 
+static void handle_thread_stack_size(const char *arg)
+{
+	/* Ignore */
+}
+
+static void handle_print_compilation(void)
+{
+	opt_print_compilation = true;
+}
+
 struct option {
 	const char *name;
 
@@ -892,28 +910,23 @@ const struct option options[] = {
 	DEFINE_OPTION("version",		handle_version),
 	DEFINE_OPTION("h",			handle_help),
 	DEFINE_OPTION("help",			handle_help),
-
-	DEFINE_OPTION_ADJACENT_ARG("D",		handle_define),
-	DEFINE_OPTION_ADJACENT_ARG("Xmx",	handle_max_heap_size),
-
-	DEFINE_OPTION_ARG("classpath",		handle_classpath),
-	DEFINE_OPTION_ARG("bootclasspath",	handle_bootclasspath),
-	DEFINE_OPTION_ARG("cp",			handle_classpath),
-	DEFINE_OPTION_ARG("jar",		handle_jar),
+	DEFINE_OPTION("client",			handle_client),
+	DEFINE_OPTION("server",			handle_server),
 	DEFINE_OPTION("verbose:gc",		handle_verbose_gc),
 
 	DEFINE_OPTION("Xmaps",			handle_maps),
 	DEFINE_OPTION("Xnewgc",			handle_newgc),
 	DEFINE_OPTION("Xnogc",			handle_nogc),
+	DEFINE_OPTION("Xnosystemclassloader",	handle_no_system_classloader),
 	DEFINE_OPTION("Xperf",			handle_perf),
 	DEFINE_OPTION("Xssa",			handle_ssa),
-
-	DEFINE_OPTION_ARG("Xtrace:method",	handle_trace_method),
-	DEFINE_OPTION_ARG("Xtrace:gate",	handle_trace_gate),
+	DEFINE_OPTION("Xnoic",			handle_no_ic),
+	DEFINE_OPTION("Xint",			handle_int),
 
 	DEFINE_OPTION("Xtrace:asm",		handle_trace_asm),
 	DEFINE_OPTION("Xtrace:bytecode",	handle_trace_bytecode),
 	DEFINE_OPTION("Xtrace:bytecode-offset",	handle_trace_bytecode_offset),
+	DEFINE_OPTION("Xtrace:verifier",	handle_trace_verifier),
 	DEFINE_OPTION("Xtrace:classloader",	handle_trace_classloader),
 	DEFINE_OPTION("Xtrace:compile",		handle_trace_compile),
 	DEFINE_OPTION("Xtrace:exceptions",	handle_trace_exceptions),
@@ -922,6 +935,21 @@ const struct option options[] = {
 	DEFINE_OPTION("Xtrace:itable",		handle_trace_itable),
 	DEFINE_OPTION("Xtrace:jit",		handle_trace_jit),
 	DEFINE_OPTION("Xtrace:trampoline",	handle_trace_trampoline),
+	DEFINE_OPTION("Xtrace:vtable",		handle_trace_vtable),
+
+	DEFINE_OPTION_ARG("classpath",		handle_classpath),
+	DEFINE_OPTION_ARG("bootclasspath",	handle_bootclasspath),
+	DEFINE_OPTION_ARG("cp",			handle_classpath),
+	DEFINE_OPTION_ARG("jar",		handle_jar),
+	DEFINE_OPTION_ARG("Xtrace:method",	handle_trace_method),
+	DEFINE_OPTION_ARG("Xtrace:gate",	handle_trace_gate),
+
+	DEFINE_OPTION_ADJACENT_ARG("Xbootclasspath/a:",	handle_bootclasspath_append),
+	DEFINE_OPTION_ADJACENT_ARG("D",		handle_define),
+	DEFINE_OPTION_ADJACENT_ARG("Xmx",	handle_max_heap_size),
+	DEFINE_OPTION_ADJACENT_ARG("Xss",	handle_thread_stack_size),
+
+	DEFINE_OPTION("XX:+PrintCompilation",	handle_print_compilation),
 };
 
 static const struct option *get_option(const char *name)
@@ -950,7 +978,7 @@ static void parse_options(int argc, char *argv[])
 
 		const struct option *opt = get_option(argv[optind] + 1);
 		if (!opt) {
-			fprintf(stderr, "%s: unrecognized option '%s'\n", exe_name, argv[optind]);
+			fprintf(stderr, "%s: unrecognized option '%s'\n", program_name, argv[optind]);
 			usage(stderr, EXIT_FAILURE);
 		}
 
@@ -981,7 +1009,7 @@ static void parse_options(int argc, char *argv[])
 		if (optind >= argc)
 			usage(stderr, EXIT_FAILURE);
 
-		classname = argv[optind++];
+		classname = dots_to_slash(argv[optind++]);
 	}
 
 	if (optind < argc) {
@@ -990,22 +1018,16 @@ static void parse_options(int argc, char *argv[])
 	}
 }
 
-static struct vm_object *get_system_class_loader(void)
-{
-	if (vm_class_ensure_init(vm_java_lang_ClassLoader))
-		return NULL;
-
-	return vm_call_method_object(vm_java_lang_ClassLoader_getSystemClassLoader);
-}
-
 static int
 do_main_class(void)
 {
-	struct vm_object *loader;
+	struct vm_object *loader = NULL;
 
-	loader = get_system_class_loader();
-	if (!loader)
-		return -1;
+	if (use_system_classloader) {
+		loader = get_system_class_loader();
+		if (!loader)
+			return -1;
+	}
 
 	if (exception_occurred()) {
 		return -1;
@@ -1049,9 +1071,14 @@ do_main_class(void)
 		array_set_field_object(args, i, arg);
 	}
 
-	void (*main_method_trampoline)(void *)
-		= vm_method_trampoline_ptr(vmm);
-	main_method_trampoline(args);
+	if (opt_interp_only) {
+		vm_interp_method(vmm, args);
+	} else {
+		void (*java_main)(void *);
+
+		java_main = vm_method_trampoline_ptr(vmm);
+		java_main(args);
+	}
 
 	return 0;
 }
@@ -1107,6 +1134,9 @@ static bool init_classpath(void)
 	const char *boot_cp = system_property_get("java.boot.class.path");
 	add_system_property_const("sun.boot.class.path", boot_cp);
 
+	if (bootclasspath_append)
+		bootclasspath_parse_and_append(bootclasspath_append);
+
 	/* Search $CLASSPATH last. */
 	char *classpath = getenv("CLASSPATH");
 	if (classpath)
@@ -1138,7 +1168,9 @@ main(int argc, char *argv[])
 {
 	int status = EXIT_FAILURE;
 
-	exe_name = argv[0];
+	program_name = argv[0];
+
+	atexit(vm_atexit);
 
 #ifndef NDEBUG
 	/* Make stdout/stderr unbuffered; it really helps debugging! */
@@ -1152,6 +1184,7 @@ main(int argc, char *argv[])
 	}
 
 	arch_init();
+	init_string_intern();
 	init_literals_hash_map();
 	init_system_properties();
 
@@ -1190,7 +1223,7 @@ main(int argc, char *argv[])
 	}
 
 	if (preload_vm_classes()) {
-		NOT_IMPLEMENTED;
+		fprintf(stderr, "Unable to preload system classes\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -1202,19 +1235,19 @@ main(int argc, char *argv[])
 
 	switch (operation) {
 	case OPERATION_MAIN_CLASS:
-		do_main_class();
+		status = do_main_class();
 		break;
 	case OPERATION_JAR_FILE:
-		do_jar_file();
+		status = do_jar_file();
 		break;
 	}
 
 out_check_exception:
 	if (exception_occurred()) {
 		vm_print_exception(exception_occurred());
+		status = EXIT_FAILURE;
 		goto out;
 	}
-	status = EXIT_SUCCESS;
 
 	vm_thread_wait_for_non_daemons();
 out:
